export class AnnotationManager {
  documentViewer: any

  constructor(documentViewer: any) {
    this.documentViewer = documentViewer
  }

  async createMeasurementAnnotation(annotation: any) {
    this.documentViewer.annotationHistory.push(annotation)
    if (!Array.isArray(annotation)) {
      annotation = [annotation]
    }
    for (let i = 0; i < annotation.length; i++) {
      const annot = annotation[i]
      if (Number(annot.pageIndex) + 1 > this.documentViewer.pagesCount) continue

      !annot.operate && (annot.operate = 'add-annot')
      !annot.author && (annot.author = this.documentViewer.annotator)

      this.documentViewer.annotations ||= {}
      this.documentViewer.pushAnnotations(annot)

      const pagePtr = this.documentViewer.pagesPtr[annot.pageIndex].pagePtr
      const result = await this.documentViewer.messageHandler.sendWithPromise('CreateMeasurementAnnotation', {
        doc: this.documentViewer.doc,
        pagePtr,
        annotation: annot
      })

      if (result && result.code) {
        this.documentViewer.eventBus.dispatch('message', {
          type: 'ERROR',
          message: result.message
        })
        console.log(result.message)
        return result
      }

      const { annotPtr, rulerPtr } = result

      annot.doc = this.documentViewer.doc
      annot.pagePtr = pagePtr
      annot.annotPtr = annotPtr
      annot.rulerPtr = rulerPtr
    }
    return annotation
  }

  async editMeasurementAnnotation(annotation: any) {
    if (!annotation) return
    await this.documentViewer.messageHandler.sendWithPromise('EditMeasurementAnnotation', annotation)
  }

  // async getMeasurementData(annotation: any) {
  //   const { annotPtr, rulerPtr } = annotation
  //   if (!annotPtr || !rulerPtr) return

  //   const result = await this.documentViewer.messageHandler.sendWithPromise('GetMeasurementData', {
  //     annotPtr,
  //     rulerPtr
  //   })

  //   if (result && result.code) {
  //     this.documentViewer.eventBus.dispatch('message', {
  //       type: 'ERROR',
  //       message: result.message
  //     })
  //     console.log(result.message)
  //   }
  //   return result
  // }

  drawAnnotationsFromList(annotationList: object[], show: boolean = true) {
    if (!Array.isArray(annotationList)) {
      annotationList = [annotationList]
    }
    for (let i = 0; i < annotationList.length; i++) {
      this.documentViewer.pdfViewer && this.documentViewer.pdfViewer.renderAnnotation(annotationList[i], show)
    }
  }
}
