import Base from './base';
import { onClickOutsideUp } from '../ui_utils'
import { getClickPoint, createSvg, createElement } from './utils';
import { MARGIN_DISTANCE } from '../../constants'
import ArrowHelper from './arrow'

export default class Line extends Base {
  constructor ({
    annotation,
    layer,
    show = false,
    enableReply = true
  }) {
    super({
      annotation,
      layer,
    })
    this.enableReply = enableReply

    this.hidden = true
    this.initial = false

    this.start = null
    this.end = null

    this.newStart = null
    this.newEnd = null

    this.svgElement = null

    this.startCircle = null
    this.middleCircle = null
    this.endCircle = null

    this.show = show
    this.ratio = window.devicePixelRatio || 1
    this.decimalPlaces = 0 // 保留小数位
    this.isPopupShow = false

    this.onMousedown = this.handleMouseDown.bind(this)
    this.onMouseup = this.handleMouseUp.bind(this)
    this.onMousemove = this.handleMouseMove.bind(this)
    this.onDelete = this.handleDelete.bind(this)
    this.onOpenReply = this.openReply.bind(this)
    this.onKeydown = this.handleKeydown.bind(this)
    this.handlePropertyPanel = this.handlePropertyPanel.bind(this)
    this.onHandlePopup = this.handlePopup.bind(this)
    this.onKeydown = this.handleKeyDown.bind(this)
    
    this.annotation.measure ? this.renderMeasure() : this.render()
  }

  render () {
    const annotation = this.annotation
    const { start, end } = this.getActualPoint(this.layer.viewport.scale)
    this.start = start
    this.end = end
    const actualbdwidth = (this.annotation.borderWidth || 2) * this.layer.viewport.scale
    const innerRect = this.calculate(start, end)
    let arrowLength = Math.max(actualbdwidth * 3, 12 * this.layer.viewport.scale)

    if (!annotation.borderColor) {
      annotation.borderColor = annotation.color || '#FF0000'
    }

    const rect = {
      left: innerRect.left - arrowLength,
      top: innerRect.top - arrowLength,
      width: innerRect.width + 2 * arrowLength,
      height: innerRect.height + 2 * arrowLength
    }
    if (!rect.width && !rect.height) return

    this.svgElement = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    this.svgElement.setAttribute("viewBox", `${rect.left} ${rect.top} ${rect.width} ${rect.height}`)
    let line=document.createElementNS("http://www.w3.org/2000/svg",'line')

    line.setAttribute("x1", this.start.x)
    line.setAttribute("y1", this.start.y)
    line.setAttribute("x2", this.end.x)
    line.setAttribute("y2", this.end.y)
    line.setAttribute("stroke", annotation.borderColor)
    line.setAttribute("stroke-width", actualbdwidth)
    line.setAttribute('mainLine', '')
    if (annotation.opacity) {
      line.setAttribute("stroke-opacity", annotation.opacity)
    }
    
    this.svgElement.append(line)

    if (this.annotation.head || this.annotation.tail) {
      let arrowBuild = new ArrowHelper(this.start, this.end)
      let arrowUi = []
      if (this.annotation.head && this.annotation.head === "OpenArrow") {
        let arrowHead=arrowBuild.getStartArrow(30, arrowLength, false)
        if(arrowHead)
        {
          arrowHead.setAttribute('head','')
          arrowUi.push(arrowHead)
        }
      }

      if (this.annotation.tail && this.annotation.tail === "OpenArrow") {
        let arrowTail=arrowBuild.getEndArrow(30, arrowLength, false)
        if(arrowTail)
        {
          arrowTail.setAttribute('tail','')
          arrowUi.push(arrowTail)
        }
      }
      arrowUi.forEach(element => {
        if (element) {
          element.setAttribute("stroke", annotation.borderColor)
          element.setAttribute("stroke-width", actualbdwidth)
          element.setAttribute("fill", "none")
          if(annotation.opacity) {
            element.setAttribute("stroke-opacity", annotation.opacity)
          }
          this.svgElement.append(element)
        }
      });
    }

    this.annotationContainer = document.createElement('div')
    this.annotationContainer.append(this.svgElement)
    this.annotationContainer.id = annotation.name
    this.annotationContainer.className = 'annotation'
    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'
    this.svgElement.addEventListener('mousedown', this.handleClick.bind(this))

    this.layer.div.append(this.annotationContainer)

    this.outerLineContainer = document.createElement('div')
    this.outerLineContainer.className = 'outline-container'
    this.deletetButton = createSvg(
      "svg",
      {
        width: "30",
        height: "30",
        viewBox: "0 0 30 30",
        fill: "none",
        class: 'delete-button'
      },
      {
        width: '30px',
        height: '30px'
      }
    );
    this.deletetButton.addEventListener('click', this.onDelete)
    this.deletetButton.innerHTML = this.deleteSvgStr
    
    this.deletetButton.style.left = (rect.left + rect.width + MARGIN_DISTANCE - 30) + 'px'
    this.deletetButton.style.top = (rect.top - MARGIN_DISTANCE - 30) + 'px'

    this.replyButton = createElement(
      'div',
      {
        width: '30px',
        height: '30px',
        background: '#DDE9FF',
        borderRadius: '2px',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center'
      },
      {
        class: 'reply-button'
      }
    );
    this.replyButton.addEventListener('click', this.onOpenReply)
    this.replyButton.innerHTML = this.replySvgStr
    this.replyButton.style.left = (rect.left + rect.width + MARGIN_DISTANCE - 65) + 'px'
    this.replyButton.style.top = (rect.top - MARGIN_DISTANCE - 30) + 'px'
    
    const outerLine = document.createElementNS("http://www.w3.org/2000/svg", "svg");
    
    outerLine.setAttribute("data-id", "move");
    outerLine.setAttribute("height", rect.height);
    outerLine.setAttribute("viewBox", `${rect.left} ${rect.top} ${rect.width + MARGIN_DISTANCE * 2} ${rect.height + MARGIN_DISTANCE * 2}`)
    outerLine.style.position = 'absolute'
    outerLine.style.zIndex = 1
    outerLine.style.left = `${rect.left - MARGIN_DISTANCE}px`
    outerLine.style.top = `${rect.top - MARGIN_DISTANCE}px`
    outerLine.style.width = `${rect.width + MARGIN_DISTANCE * 2}px`
    outerLine.style.height = `${rect.height + MARGIN_DISTANCE * 2}px`
    this.outerLine = outerLine
    
    this.startCircle = createSvg(
      "circle",
      {
        class: "start",
        'data-id': "start",
        fill: "#1460F3",
        cx: start.x + MARGIN_DISTANCE,
        cy: start.y + MARGIN_DISTANCE,
        r: 6
      }
    );
    this.endCircle = createSvg(
      "circle",
      {
        class: "end",
        'data-id': "end",
        fill: "#1460F3",
        cx: end.x + MARGIN_DISTANCE,
        cy: end.y + MARGIN_DISTANCE,
        r: 6
      }
    );
    this.moveLine = createSvg(
      "line",
      {
        class: 'move',
        'data-id': "move",
        x1: start.x + MARGIN_DISTANCE,
        y1: start.y + MARGIN_DISTANCE,
        x2: end.x + MARGIN_DISTANCE,
        y2: end.y + MARGIN_DISTANCE,
        fill: 'transparent',
        stroke: 'transparent',
        'stroke-width': 5
      }
    )
    this.outerLine.append(this.moveLine)
    this.outerLine.append(this.startCircle)
    this.outerLine.append(this.endCircle)
    this.outerLineContainer.append(this.outerLine)
    const isAnnotationPopup = this.layer.documentViewer.isAnnotationPopup
    isAnnotationPopup && this.outerLineContainer.append(this.deletetButton);
    (isAnnotationPopup && this.enableReply) && this.outerLineContainer.append(this.replyButton)
    this.initial = true

    if (this.show) {
      !this.layer.addMode && this.selectAnnotation()
      this.show = false
    }
  }

  async renderMeasure () {
    const annotation = this.annotation
    const scale = this.layer.viewport.scale
    const { start, end } = this.getActualPoint(scale)
    this.start = start
    this.end = end

    if (annotation.precision) {
      if (annotation.precision > 1) {
        annotation.precision = Math.pow(10, -Math.floor(Math.log10(annotation.precision)))
      }
      this.decimalPlaces = Math.abs(annotation.precision).toString().split('.')[1]?.length || 0
    }
    if (!annotation.borderColor) {
      annotation.borderColor = annotation.color || '#FF0000'
    }
    if (!annotation.factor) {
      annotation.factor = await this.layer.documentViewer.messageHandler.sendWithPromise('GetFactor', annotation.rulerPtr)
    }
    if (!annotation.X) {
      const initPoint = this.getInitialPoint()
      annotation.X = (Math.abs(initPoint.end.x - initPoint.start.x) * annotation.factor).toFixed(this.decimalPlaces)
    }
    if (!annotation.Y) {
      const initPoint = this.getInitialPoint()
      annotation.Y = (Math.abs(initPoint.end.y - initPoint.start.y) * annotation.factor).toFixed(this.decimalPlaces)
    }
    if (!annotation.angle) {
      const angleDeg = Math.atan2(end.y - start.y, end.x - start.x) / Math.PI * 180
      const angle = Math.abs(angleDeg) > 90 ? 180 - Math.abs(angleDeg) : angleDeg
      const formatAngle = Math.abs(angle).toFixed(this.decimalPlaces) * 1
      annotation.angle = formatAngle
    }
    if (!annotation.fontStyle) {
      annotation.fontStyle = 'regular'
    }
    if (!annotation.contents) {
      await this.getMeasureData()
    }

    const leadLength = annotation.leadLength * scale
    const leadExtension = annotation.leadExtension * scale
    const dx = end.x - start.x
    const dy = end.y - start.y
    const length = Math.sqrt(dx * dx + dy * dy)
    const unitX = dx / length
    const unitY = dy / length
    const perpX = -unitY
    const perpY = unitX

    const endLead1 = {
      x: start.x + perpX * leadLength,
      y: start.y + perpY * leadLength
    }
    const endLead2 = {
      x: end.x + perpX * leadLength,
      y: end.y + perpY * leadLength
    }
    const endLeadExtension1 = {
      x: start.x - perpX * leadExtension,
      y: start.y - perpY * leadExtension
    }
    const endLeadExtension2 = {
      x: end.x - perpX * leadExtension,
      y: end.y - perpY * leadExtension
    }

    const innerRect = this.maxRect([endLead1, endLead2, endLeadExtension1, endLeadExtension2])
    const actualbdwidth = (this.annotation.borderWidth || 2) * scale
    let arrowLength = Math.max(actualbdwidth * 3, 10 * scale)

    const rect = {
      left: innerRect.left - arrowLength,
      top: innerRect.top - arrowLength,
      width: innerRect.width + 2 * arrowLength,
      height: innerRect.height + 2 * arrowLength
    }

    if (!rect.width && !rect.height) return
    this.rect = rect

    this.svgElement = document.createElementNS("http://www.w3.org/2000/svg", "svg");
    this.svgElement.setAttribute("viewBox", `${rect.left} ${rect.top} ${rect.width} ${rect.height}`)
    this.svgElement.style.pointerEvents = 'all'
    this.svgElement.style.overflow = 'visible'
    if (annotation.opacity) {
      this.svgElement.style.opacity = annotation.opacity
    }
    
    const midpoint = {
      x: (start.x + end.x) / 2,
      y: (start.y + end.y) / 2
    }
    const angleDeg = Math.atan2(end.y - start.y, end.x - start.x) / Math.PI * 180
    const angle = Math.abs(angleDeg) > 90 ? angleDeg - 180 : angleDeg

    this.textEle = createSvg('text', {
      fill: annotation.color || annotation.borderColor,
      'text-anchor': 'middle',
      'dominant-baseline': 'middle',
      'font-size': annotation.fontSize,
      x: midpoint.x,
      y: midpoint.y,
      transform: `rotate(${angle} ${midpoint.x} ${midpoint.y})`,
    }, {
      pointerEvents: 'none'
    })
    this.textEle.textContent = annotation.contents
    this.svgElement.append(this.textEle)

    const textGap = this.getTextWidth()

    const line1End = {
      x: start.x + unitX * (length / 2 - textGap / 2),
      y: start.y + unitY * (length / 2 - textGap / 2),
    }
    const line2Start = {
      x: start.x + unitX * (length / 2 + textGap / 2),
      y: start.y + unitY * (length / 2 + textGap / 2),
    }

    let dashString = ''
    const scaleValue = Math.ceil(this.ratio * this.layer.viewport.scale)
    if (annotation.dashes.length === 1 || (annotation.dashes.length === 2 && annotation.dashes[0] === annotation.dashes[1])) {
      dashString = Math.ceil(annotation.dashes[0] * scaleValue) + ''
    } else if (annotation.dashes.length > 2) {
      dashString = annotation.dashes.map(dash => Math.ceil(dash * scaleValue)).join(', ')
    }

    const lines = [{ // 主体线段1（起始点到中间点）
      'startLine': '',
      x1: start.x,
      y1: start.y,
      x2: line1End.x,
      y2: line1End.y,
      'stroke-dasharray': dashString
    }, { // 主体线段2（中间点到结束点）
      'endLine': '',
      x1: line2Start.x,
      y1: line2Start.y,
      x2: end.x,
      y2: end.y,
      'stroke-dasharray': dashString
    }, { // 起始点引线
      'startLead': '',
      x1: start.x,
      y1: start.y,
      x2: endLead1.x,
      y2: endLead1.y,
    }, { // 结束点引线
      'endLead': '',
      x1: end.x,
      y1: end.y,
      x2: endLead2.x,
      y2: endLead2.y,
    }, { // 起始点延长引线
      'startLeadExtension': '',
      x1: start.x,
      y1: start.y,
      x2: endLeadExtension1.x,
      y2: endLeadExtension1.y,
    }, { // 结束点延长引线
      'endLeadExtension': '',
      x1: end.x,
      y1: end.y,
      x2: endLeadExtension2.x,
      y2: endLeadExtension2.y,
    }]
    
    lines.forEach(line => {
      const lineEle = createSvg('line', {
        ...line,
        stroke: annotation.borderColor,
        'stroke-width': actualbdwidth,
      }, {
        'stroke-linecap': 'butt'
      })
      this.svgElement.append(lineEle)
    })

    if (this.annotation.head || this.annotation.tail) {
      let arrowBuild = new ArrowHelper(this.start, this.end)
      let arrowUi = []
      if (this.annotation.head && this.annotation.head === "OpenArrow") {
        let arrowHead = arrowBuild.getStartArrow(45, arrowLength)
        if (arrowHead) {
          arrowHead.setAttribute('head','')
          arrowUi.push(arrowHead)
        }
      }

      if (this.annotation.tail && this.annotation.tail === "OpenArrow") {
        let arrowTail = arrowBuild.getEndArrow(45, arrowLength)
        if (arrowTail) {
          arrowTail.setAttribute('tail','')
          arrowUi.push(arrowTail)
        }
      }
      arrowUi.forEach(element => {
        if (element) {
          element.setAttribute("stroke", annotation.borderColor)
          element.setAttribute("stroke-width", actualbdwidth)
          element.setAttribute("fill", "none")
          this.svgElement.append(element)
        }
      })
    }

    this.annotationContainer = document.createElement('div')
    this.annotationContainer.append(this.svgElement)
    this.annotationContainer.id = annotation.name
    this.annotationContainer.className = 'annotation'
    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'
    this.svgElement.addEventListener('mousedown', this.handleClick.bind(this))

    this.layer.div.append(this.annotationContainer)

    this.outerLineContainer = document.createElement('div')
    this.outerLineContainer.className = 'outline-container'
    
    const outerLine = document.createElementNS("http://www.w3.org/2000/svg", "svg");
    outerLine.setAttribute("data-id", "move");
    outerLine.setAttribute("width", rect.width);
    outerLine.setAttribute("height", rect.height);
    outerLine.setAttribute("viewBox", `${rect.left} ${rect.top} ${rect.width} ${rect.height}`)
    outerLine.style.position = 'absolute'
    outerLine.style.zIndex = 1
    outerLine.style.left = `${rect.left}px`
    outerLine.style.top = `${rect.top}px`
    outerLine.style.width = `${rect.width}px`
    outerLine.style.height = `${rect.height}px`
    outerLine.style.border = `${parseFloat(scale.toFixed(1))}px dashed #1460F3`
    outerLine.style.boxSizing = 'content-box'
    outerLine.style.pointerEvents = 'all'
    outerLine.style.cursor = 'all-scroll'
    this.outerLine = outerLine
    const direction = this.annotation.leadLength > this.annotation.leadExtension

    this.startCircle = createSvg(
      "circle",
      {
        class: "start",
        'data-id': "start",
        fill: "#1460F3",
        cx: direction ? endLead1.x : endLeadExtension1.x,
        cy: direction ? endLead1.y : endLeadExtension1.y,
        r: 6
      }, {
        cursor: 'default'
      }
    );
    this.middleCircle = createSvg(
      "circle",
      {
        class: "middle",
        'data-id': "middle",
        fill: "#1460F3",
        cx: midpoint.x,
        cy: midpoint.y,
        r: 6
      }, {
        cursor: 'default'
      }
    );
    this.endCircle = createSvg(
      "circle",
      {
        class: "end",
        'data-id': "end",
        fill: "#1460F3",
        cx: direction ? endLead2.x : endLeadExtension2.x,
        cy: direction ? endLead2.y : endLeadExtension2.y,
        r: 6
      }, {
        cursor: 'default'
      }
    );
    this.moveLine = createSvg(
      "line",
      {
        class: 'move',
        'data-id': "move",
        x1: start.x,
        y1: start.y,
        x2: end.x,
        y2: end.y,
        fill: 'transparent',
        stroke: 'transparent',
        'stroke-width': 5
      }
    )
    this.outerLine.append(this.moveLine)
    this.outerLine.append(this.startCircle)
    this.outerLine.append(this.middleCircle)
    this.outerLine.append(this.endCircle)
    this.outerLineContainer.append(this.outerLine)
    this.initial = true

    if (this.show) {
      !this.layer.addMode && this.selectAnnotation()
      this.show = false
    }
  }

  selectAnnotation () {
    if (this.layer.annotationStore.selectedElementName === this.annotation.name) return
    this.handleClick()
  }

  handleClick (event) {
    if (!this.hidden || this.layer.annotationStore.creating || ['redaction', 'remove'].includes(this.layer.tool) || document.querySelector('.compare-document-container').contains(event?.target)) return
    this.hidden = false
    this.updateTool()

    if (this.annotation.measure) {
      this.layer.eventBus._on('measurePropertyPanelChanged', this.handlePropertyPanel)
      this.layer.eventBus._on('measurementPopupClicked', this.onHandlePopup)
    }
    document.addEventListener('keydown', this.onKeydown)

    const ignoreList = [
      this.svgElement,
      this.outerLine,
      document.querySelector('.toggle-button'),
      document.getElementById('propertyPanelButton'),
      document.querySelector('.measure-panel'),
      document.querySelector('.measurement-board'),
      document.querySelector('.measurements-settings'),
      document.querySelector('.measurement-popup')
    ]
    this.deletetButton && ignoreList.push(this.deletetButton)
    this.replyButton && ignoreList.push(this.replyButton)
    onClickOutsideUp(ignoreList, this.handleOutside.bind(this))
  }

  handleOutside () {
    if (!this.hidden && this.annotation.measure && this.layer.annotationStore.selectedElementName === this.annotation.name) {
      this.layer.eventBus.dispatch('changeRightPanelBtnDisabled')
      this.layer.eventBus.dispatch('measurePropertyChange', { isOpen: false })
      this.hidePopup()
    }

    this.hidden = true
    if (this.layer.annotationStore.selectedElementName === this.annotation.name) {
      this.layer.annotationStore.selectedElementName = null
    }

    document.removeEventListener('keydown', this.onKeydown)
    this.outerLine?.removeEventListener('mousedown', this.onMousedown)
    this.outerLine?.removeEventListener('touchstart', this.onMousedown)
    this.outerLineContainer?.remove()

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    scrollElement.style.touchAction = 'auto'

    if (this.annotation.measure) {
      this.layer.eventBus._off('measurePropertyPanelChanged', this.handlePropertyPanel)
      this.layer.eventBus._off('measurementPopupClicked', this.onHandlePopup)
    }
  }

  getActualPoint (s) {
    const annotation = this.annotation
    if (!annotation.linePoints && annotation.start && annotation.end) {
      annotation.linePoints = annotation.start + ',' + annotation.end
    }
    let [startX, startY, endX, endY] = annotation.linePoints

    let x1, y1, x2, y2
    x1 = startX * s
    y1 = startY * s
    x2 = endX * s
    y2 = endY * s

    return {
      start: {
        x: x1,
        y: y1,
      },
      end: {
        x: x2,
        y: y2,
      },
    }
  }

  getInitialPoint () {
    const s = this.layer.viewport.scale
    const startX = this.start.x
    const startY = this.start.y
    const endX = this.end.x
    const endY = this.end.y

    let x1, y1, x2, y2
    x1 = startX / s
    y1 = startY / s
    x2 = endX / s
    y2 = endY / s
    return {
      start: {
        x: x1,
        y: y1,
      },
      end: {
        x: x2,
        y: y2,
      },
    }
  }

  calculate (start, end) {
    const initRect = this.rectCalc({ start, end });
    const actualbdwidth = (this.annotation.borderWidth || 2) * this.layer.viewport.scale
    if (this.annotation.arrow) {
      return {
        top: initRect.top - 5 * 2,
        left: initRect.left - 5 * 2,
        width: initRect.width + 5 * 4,
        height: initRect.height + 5 * 4,
      }
    } else {
      return {
        top: initRect.top - actualbdwidth / 2,
        left: initRect.left - actualbdwidth / 2,
        width: initRect.width + actualbdwidth,
        height: initRect.height + actualbdwidth,
      }
    }
  }

  rectCalc ({ start, end }) {
    return {
      top: start.y < end.y ? start.y : end.y,
      left: start.x < end.x ? start.x : end.x,
      width: Math.abs(end.x - start.x),
      height: Math.abs(end.y - start.y),
    };
  };

  maxRect(points) {
    const minX = Math.min(points[0].x, points[1].x, points[2].x, points[3].x);
    const minY = Math.min(points[0].y, points[1].y, points[2].y, points[3].y);
    const maxX = Math.max(points[0].x, points[1].x, points[2].x, points[3].x);
    const maxY = Math.max(points[0].y, points[1].y, points[2].y, points[3].y);

    return  {
      left: minX,
      top: minY,
      width: maxX - minX,
      height: maxY - minY
    }
  }

  updateTool () {
    if (!this.initial) return
    if (this.hidden) {
      if (this.layer.annotationStore.selectedElementName === this.annotation.name) {
        this.layer.annotationStore.selectedElementName = null
      }
      this.outerLineContainer.remove()
      document.removeEventListener('keydown', this.onKeydown)
    } else {
      if (this.layer.annotationStore.selectedElementName !== this.annotation.name) {
        const annotation = this.annotation
        this.layer.annotationStore.selectedElementName = annotation.name

        const annotationData = {
          ...annotation
        }
        delete annotationData.annotPtr
        delete annotationData.pagePtr
        delete annotationData.operate
        delete annotationData.targetPage
        delete annotationData.arrow

        this.layer.eventBus.dispatch('annotationSelected', annotationData)
      }
      this.layer.div.append(this.outerLineContainer)
      this.outerLine.addEventListener('mousedown', this.onMousedown)
      this.outerLine.addEventListener('touchstart', this.onMousedown)
      document.addEventListener('keydown', this.onKeydown)

      if (this.annotation.measure) {
        this.layer.eventBus.dispatch('changeRightPanelBtnDisabled')
        this.layer.eventBus.dispatch('measurePropertyChange', this.annotation)
        this.layer.eventBus.dispatch('updateMeasureData', {
          scale: this.annotation.scale,
          precision: this.annotation.precision,
          length: this.annotation.measureResult,
          angle: this.annotation.angle,
          X: this.annotation.X,
          Y: this.annotation.Y,
        })
        this.showPopup()
      }

      const documentViewer = this.layer.documentViewer
      const scrollElement = documentViewer.getScrollViewElement()
      scrollElement.style.touchAction = 'none'
    }
  }

  handleMouseDown (event) {
    if (event.button !== 0 && event.type === 'mousedown') return
    if (this.layer.tool) {
      event.stopPropagation()
    }
    const operatorId = event.target.getAttribute('data-id')
  
    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    this.scrollTop = scrollElement.scrollTop
    this.scrollLeft = scrollElement.scrollLeft

    const { pageX, pageY } = getClickPoint(event)
    this.startState = {
      operator: operatorId,
      clickX: pageX,
      clickY: pageY,
    }
    this.initialPoints = { start: this.start, end: this.end }
    this.initLeadLength = this.annotation.leadLength
    this.initLeadExtension = this.annotation.leadExtension
    this.hasUpdatedInitialRect = false

    document.addEventListener('mousemove', this.onMousemove)
    document.addEventListener('mouseup', this.onMouseup)
    document.addEventListener('touchmove', this.onMousemove)
    document.addEventListener('touchend', this.onMouseup)
  }

  handleMouseMove (event) {
    if (event.button !== 0 && event.type === 'mousemove') return
    this.hidePopup()
    
    if (event.type === 'touchmove') {
      this.layer.pageDiv.style.touchAction = 'none'
    }
    this.moving = true
    let { pageX, pageY } = getClickPoint(event)

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    const scrollTop = scrollElement.scrollTop
    const scrollLeft = scrollElement.scrollLeft
    pageX = pageX + scrollLeft - this.scrollLeft
    pageY = pageY + scrollTop - this.scrollTop

    const start = this.start
    const end = this.end
    const startState = this.startState
    const { width, height, scale } = this.layer.viewport

    if (startState.operator === 'start') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      x1 = Math.min(width, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(height, y1)
      y1 = Math.max(0, y1)

      this.update({
        start: { x: x1, y: y1 },
        end,
      });
    } else if (startState.operator === 'middle') {
      let margin = document.querySelector('.document-container')?.style.marginLeft || 0
      margin = parseFloat(margin)
      pageX = pageX + this.scrollLeft - margin
      pageY += this.scrollTop
      const pagePoint = this.layer.documentViewer.windowToPage({ x: pageX, y: pageY }, this.layer.page + 1, false)
      // 步骤 1：计算两点的中点 midpoint
      const midpoint = {
        x: (start.x + end.x) / 2,
        y: (start.y + end.y) / 2
      }
      // 步骤 2：计算向量 AB 和垂直向量 perpVec
      const vectorAB = {
        x: end.x - start.x,
        y: end.y - start.y
      }
      const perpVec = {
        x: -vectorAB.y,  // 垂直向量的 x 分量
        y: vectorAB.x    // 垂直向量的 y 分量
      }
      // 步骤 3：计算向量 AC
      const vectorAC = {
        x: pagePoint.x - midpoint.x,
        y: pagePoint.y - midpoint.y
      }
      // 步骤 4：计算 AC 在垂直向量 perpVec 方向上的投影
      const dotProduct = vectorAC.x * perpVec.x + vectorAC.y * perpVec.y  // AC 与 perpVec 的点积
      const perpVecMagSq = perpVec.x * perpVec.x + perpVec.y * perpVec.y // perpVec 的模的平方
      // 计算投影的标量因子
      const projectionFactor = dotProduct / perpVecMagSq

      let x1 = start.x + projectionFactor * perpVec.x
      let y1 = start.y + projectionFactor * perpVec.y
      let x2 = end.x + projectionFactor * perpVec.x
      let y2 = end.y + projectionFactor * perpVec.y
      x1 = Math.max(0, Math.min(width, x1))
      y1 = Math.max(0, Math.min(height, y1))
      x2 = Math.max(0, Math.min(width, x2))
      y2 = Math.max(0, Math.min(height, y2))

      if (x1 === 0 || x1 === width || y1 === 0 || y1 === height
        || x2 === 0 || x2 === width || y2 === 0 || y2 === height) return

      let dx, dy, distance
      // 计算距离的通用函数
      const calculateDistance = (x1, y1, x2, y2) => {
        const initMidX = (this.initialPoints.start.x + this.initialPoints.end.x) / 2
        const initMidY = (this.initialPoints.start.y + this.initialPoints.end.y) / 2
        dx = (x1 + x2) / 2 - initMidX
        dy = (y1 + y2) / 2 - initMidY
        distance = Math.sqrt(dx * dx + dy * dy) / this.layer.viewport.scale
      }
      calculateDistance(x1, y1, x2, y2)

      // 根据点距计算直线移动的方向，往 lead 或 leadExtension 方向
      // 若是往 lead 方向，则减小 leadLength 增大 leadExtension ；若是往 leadExtension 方向，则减小 leadExtension 增大 leadLength
      const determineDirection = () => {
        const startLead = this.svgElement.querySelector('[startLead]')
        const startLeadExtension = this.svgElement.querySelector('[startLeadExtension]')

        const startLeadPoint = {
          x: startLead.getAttribute('x2'),
          y: startLead.getAttribute('y2')
        }
        const startLeadExtPoint = {
          x: startLeadExtension.getAttribute('x2'),
          y: startLeadExtension.getAttribute('y2')
        }

        // 计算两点之间的向量
        const vector = (p1, p2) => ({ x: p2.x - p1.x, y: p2.y - p1.y })
        // 计算两个向量的点积
        const dotProduct = (v1, v2) => (v1.x * v2.x + v1.y * v2.y)

        // 计算向量
        let BA = vector(this.initialPoints.start, startLeadPoint)
        let BC = vector(this.initialPoints.start, startLeadExtPoint)
        let BnewBold = vector(this.initialPoints.start, { x: x1, y: y1 })

        // 计算点积
        let dotBA = dotProduct(BnewBold, BA)
        let dotBC = dotProduct(BnewBold, BC)

        if (dotBA > dotBC) return 'toLead'
        if (dotBC > dotBA) return 'toExtension'
      }

      let direction = determineDirection()
      // 若三点重合，无法判断准确方向。根据默认向量判断方向
      if (!direction) {
        direction = (dx > 0) ? 'toExtension' :
                    (dx < 0) ? 'toLead' :
                    (dy < 0 ? 'toLead' : 'toExtension')
      }

      // 为了不让三点重合，给了一个最小值0.1
      if (direction === 'toExtension') {
        if (this.annotation.leadExtension <= 0.1) {
          // 检查是否首次从不为0变为0
          if (this.initLeadExtension !== 0.1 && this.annotation.leadExtension === 0.1 && !this.hasUpdatedInitialRect) {
            this.initialPoints = { start: { x: x1, y: y1 }, end: { x: x2, y: y2 }}
            this.hasUpdatedInitialRect = true
            // 重新计算 distance
            calculateDistance(x1, y1, x2, y2)
          }

          this.annotation.leadLength = Math.max(this.initLeadLength + distance, 0.1)
          this.initLeadExtension = 0.1
        }
        this.annotation.leadExtension = Math.max(this.initLeadExtension - distance, 0.1)
      } else if (direction === 'toLead') {
        if (this.annotation.leadLength <= 0.1) {
          // 检查是否首次从不为0变为0
          if (this.initLeadLength !== 0.1 && this.annotation.leadLength === 0.1 && !this.hasUpdatedInitialRect) {
            this.initialPoints = { start: { x: x1, y: y1 }, end: { x: x2, y: y2 }}
            this.hasUpdatedInitialRect = true
            // 重新计算 distance
            calculateDistance(x1, y1, x2, y2)
          }

          this.annotation.leadExtension = Math.max(this.initLeadExtension + distance, 0.1)
          this.initLeadLength = 0.1
        }
        this.annotation.leadLength = Math.max(this.initLeadLength - distance, 0.1)
      }

      this.update({
        start: { x: x1, y: y1 },
        end: { x: x2, y: y2 },
      })

    } else if (startState.operator === 'end') {
      let x2 = pageX - (startState.clickX - this.end.x);
      let y2 = pageY - (startState.clickY - this.end.y);
      x2 = Math.min(width, x2)
      x2 = Math.max(0, x2)
      y2 = Math.min(height, y2)
      y2 = Math.max(0, y2)
      this.update({
        start,
        end: { x: x2, y: y2 },
      });
    } else if (startState.operator === 'move') {
      let x1, y1, x2, y2
      const documentViewer = this.layer.documentViewer

      let windowCoordinates = this.getMouseLocation(event)
      const page = documentViewer.getSelectedPage(windowCoordinates, windowCoordinates)
      if (page.first && page.first !== (this.layer.page + 1)) {
        const pagePoint = documentViewer.windowToPage(windowCoordinates, page.first, false)

        const annotationWidth = Math.abs(start.x - end.x)
        const annotationHeight = Math.abs(start.y - end.y)
        if (documentViewer.pdfViewer.spreadMode === 0 && documentViewer.pdfViewer.scrollMode === 0) {
          x1 = pageX - (startState.clickX - this.start.x)
          y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
          x2 = pageX - (startState.clickX - this.end.x)
          y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
        } else if (documentViewer.pdfViewer.spreadMode === 0 && documentViewer.pdfViewer.scrollMode === 1) {
          x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
          y1 = pageY - (startState.clickY - this.start.y)
          x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
          y2 = pageY - (startState.clickY - this.end.y)
        } else if (documentViewer.pdfViewer.spreadMode === 1 && documentViewer.pdfViewer.scrollMode === 0) {
          const minPage = Math.min(page.first, this.layer.page + 1)
          const maxPage = Math.max(page.first, this.layer.page + 1)
          if ((maxPage - minPage) > 2 || ((minPage + 1) === maxPage && minPage % 2 === 0)) {
            return
          }
          if ((minPage + 1) === maxPage) {
            x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
            y1 = pageY - (startState.clickY - this.start.y)
            x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
            y2 = pageY - (startState.clickY - this.end.y)
          } else if ((minPage + 2) === maxPage) {
            x1 = pageX - (startState.clickX - this.start.x)
            y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
            x2 = pageX - (startState.clickX - this.end.x)
            y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
          }
        } else if (documentViewer.pdfViewer.spreadMode === 2 && documentViewer.pdfViewer.scrollMode === 0) {
          const minPage = Math.min(page.first, this.layer.page + 1)
          const maxPage = Math.max(page.first, this.layer.page + 1)
          if ((maxPage - minPage) > 2 || ((minPage + 1) === maxPage && !(minPage === 1 && maxPage === 2) && maxPage % 2 === 0 || (minPage === 1 && maxPage === 3))) {
            return
          }
          if ((minPage + 2) === maxPage || (minPage === 1 && maxPage === 2)) {
            x1 = pageX - (startState.clickX - this.start.x)
            y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
            x2 = pageX - (startState.clickX - this.end.x)
            y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
          } else if ((minPage + 1) === maxPage) {
            x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
            y1 = pageY - (startState.clickY - this.start.y)
            x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
            y2 = pageY - (startState.clickY - this.end.y)
          }
        }
        this.layer = documentViewer.pdfViewer._pages[page.first - 1].compdfAnnotationLayer

        this.annotationContainer.remove()
        this.outerLineContainer.remove()
        const container = this.layer.div
        container.append(this.annotationContainer)
        container.append(this.outerLineContainer)
      } else {
        x1 = pageX - (startState.clickX - this.start.x);
        y1 = pageY - (startState.clickY - this.start.y);
        x2 = pageX - (startState.clickX - this.end.x);
        y2 = pageY - (startState.clickY - this.end.y);

        const windowPoint1 = documentViewer.pageToWindow({
          x: x1,
          y: y1
        }, this.layer.page + 1, false)
        const windowPoint2 = documentViewer.pageToWindow({
          x: x2,
          y: y2
        }, this.layer.page + 1, false)
        let x = 0, y = 0
        if (pageY > startState.clickY) {
          y = scrollElement.offsetHeight - (windowPoint2.y - scrollElement.scrollTop - scrollElement.offsetTop)
          y = y < 100 ? 5 : 0
        } else if (pageY < startState.clickY) {
          y = windowPoint1.y - scrollElement.scrollTop - scrollElement.offsetTop
          y = y < 100 ? -5 : 0
        }
        if (pageX > startState.clickX) {
          x = scrollElement.offsetWidth - (windowPoint2.x - scrollElement.scrollLeft - scrollElement.offsetLeft)
          x = x < 100 ? 5 : 0
        } else if (pageX < startState.clickX) {
          x = (windowPoint1.x - scrollElement.scrollLeft - scrollElement.offsetLeft)
          x = x < 100 ? -5 : 0
        }

        if (x || y) {
          scrollElement.scrollTop += y
          scrollElement.scrollLeft += x
        }

        const leadLength = this.annotation.leadLength * scale
        const leadExtension = this.annotation.leadExtension * scale
        const dx = x2 - x1
        const dy = y2 - y1
        const length = Math.sqrt(dx * dx + dy * dy)
        const unitX = dx / length
        const unitY = dy / length
        const perpX = -unitY
        const perpY = unitX

        const endLead1 = {
          x: x1 + perpX * leadLength,
          y: y1 + perpY * leadLength
        }
        const endLead2 = {
          x: x2 + perpX * leadLength,
          y: y2 + perpY * leadLength
        }
        const endLeadExtension1 = {
          x: x1 - perpX * leadExtension,
          y: y1 - perpY * leadExtension
        }
        const endLeadExtension2 = {
          x: x2 - perpX * leadExtension,
          y: y2 - perpY * leadExtension
        }

        const rect = {
          width: Math.abs(start.x -end.x),
          height: Math.abs(start.y -end.y)
        }
        if (Math.min(endLead1.x, endLead2.x, endLeadExtension1.x, endLeadExtension2.x) < 0
        || Math.max(endLead1.x, endLead2.x, endLeadExtension1.x, endLeadExtension2.x) > width) {
          x1 = start.x
          x2 = end.x
        } else if (x1 < x2) {
          x1 = Math.max(0, x1)
          x1 = Math.min(width - rect.width, x1)
          x2 = Math.max(rect.width, x2)
          x2 = Math.min(width, x2)
        } else {
          x2 = Math.max(0, x2)
          x2 = Math.min(width - rect.width, x2)
          x1 = Math.max(rect.width, x1)
          x1 = Math.min(width, x1)
        }

        if (Math.min(endLead1.y, endLead2.y, endLeadExtension1.y, endLeadExtension2.y) < 0
        || Math.min(endLead1.y, endLead2.y, endLeadExtension1.y, endLeadExtension2.y) > height) {
          y1 = start.y
          y2 = end.y
        } else if (y1 < y2) {
          y1 = Math.max(0, y1)
          y1 = Math.min(height - rect.height, y1)
          y2 = Math.max(rect.height, y2)
          y2 = Math.min(height, y2)
        } else {
          y2 = Math.max(0, y2)
          y2 = Math.min(height - rect.height, y2)
          y1 = Math.max(rect.height, y1)
          y1 = Math.min(height, y1)
        }
      }

      this.update({
        start: { x: x1, y: y1 },
        end: { x: x2, y: y2 },
      });
    }
    startState.clickX = pageX
    startState.clickY = pageY
  }

  handleDelete (event) {
    if (this.layer.tool && event) {
      event.stopPropagation()
    }

    this.delete()
    const annotationData = {
      type: 'delete',
      annotation: {
        operate: "del-annot",
        name: this.annotation.name,
        pageIndex: this.layer.page
      }
    }
    this.annotation.annotPtr && (annotationData.annotation.annotPtr = this.annotation.annotPtr)

    this.layer.eventBus.dispatch('annotationChanged', annotationData)
  }

  delete () {
    this.handleOutside()
    this.annotationContainer?.remove()
    this.annotation.isDelete = true
    this.layer.deleteAnnotationInLayer({
      name: this.annotation.name
    })
  }

  handleKeydown (event) {
    // if (event.key === 'Delete' || event.key === 'Backspace') {
    //   this.handleDelete()
    // }
  }


  async handleMouseUp (event) {
    if (event.button !== 0 && event.type === 'mouseup') return
    if (this.layer.tool) {
      event.stopPropagation()
    }
    if (event.type === 'touchend') {
      this.layer.pageDiv.style.touchAction = ''
    }
    this.moving = false
    this.initialPoints = null
    document.removeEventListener('mousemove', this.onMousemove)
    document.removeEventListener('mouseup', this.onMouseup)
    document.removeEventListener('touchmove', this.onMousemove)
    document.removeEventListener('touchend', this.onMouseup)
    this.scrollLeft = 0
    this.scrollTop = 0

    const annotation = this.annotation
    const { start, end } = this.getInitialPoint()

    const linePoints = [start.x, start.y, end.x, end.y]
    annotation.linePoints = linePoints
    const left = Math.min(start.x, end.x)
    const top = Math.min(start.y, end.y)
    const right = Math.max(start.x, end.x)
    const bottom = Math.max(start.y, end.y)
    const rect = {
      left: left - 6,
      top: top - 6,
      right: right + 6,
      bottom: bottom + 6
    }
    annotation.rect = rect
    this.annotation.measure && this.showPopup()

    if (annotation.pageIndex !== this.layer.page) {
      const annotationData = {
        type: 'delete',
        annotation: {
          operate: "del-annot",
          name: annotation.name,
          pageIndex: annotation.pageIndex,
          annotPtr: annotation.annotPtr
        }
      }
      this.layer.eventBus.dispatch('annotationChanged', annotationData)

      const newAnnotationData = Object.assign({}, annotation)
      delete newAnnotationData.annotPtr
      delete newAnnotationData.pagePtr
      delete newAnnotationData.name
      newAnnotationData.operate = 'add-annot'
      newAnnotationData.pageIndex = this.layer.page
      // this.layer.eventBus.dispatch('annotationChanged', {
      //   type: 'add',
      //   show: true,
      //   annotation: newAnnotationData
      // })
      const annots = await this.layer.documentViewer.annotationManager.createMeasurementAnnotation(newAnnotationData)
      this.annotation = annots[0]
      await this.layer.documentViewer.annotationManager.drawAnnotationsFromList(this.annotation)

      this.annotationContainer.remove()
      this.outerLineContainer.remove()
    } else {
      const annotationData = {
        operate: "mod-annot",
        name: annotation.name,
        pageIndex: this.layer.page,
        pagePtr: annotation.pagePtr,
        annotPtr: annotation.annotPtr,
        linePoints,
        rect
      }
      // this.layer.eventBus.dispatch('annotationChanged', {
      //   type: 'modify',
      //   annotation: annotationData
      // })
      await this.layer.documentViewer.annotationManager.editMeasurementAnnotation(annotationData)
    }
  }

  openReply () {
    this.layer.eventBus.dispatch('openAnnotationReply', this.annotation)
  }

  update ({ start, end }, isUpdateMeasure = true) {
    this.start = start
    this.end = end

    const scale = this.layer.viewport.scale
    const actualbdwidth = (this.annotation.borderWidth || 2) * scale
    let arrowLength = Math.max(actualbdwidth * 3, 10 * scale)

    let newRect, endLead1, endLead2, endLeadExtension1, endLeadExtension2
    if (this.annotation.measure) {
      const leadLength = this.annotation.leadLength * scale
      const leadExtension = this.annotation.leadExtension * scale
      const dx = end.x - start.x
      const dy = end.y - start.y
      const length = Math.sqrt(dx * dx + dy * dy)
      const unitX = dx / length
      const unitY = dy / length
      const perpX = -unitY
      const perpY = unitX

      endLead1 = {
        x: start.x + perpX * leadLength,
        y: start.y + perpY * leadLength
      }
      endLead2 = {
        x: end.x + perpX * leadLength,
        y: end.y + perpY * leadLength
      }
      endLeadExtension1 = {
        x: start.x - perpX * leadExtension,
        y: start.y - perpY * leadExtension
      }
      endLeadExtension2 = {
        x: end.x - perpX * leadExtension,
        y: end.y - perpY * leadExtension
      }

      const innerRect = this.maxRect([endLead1, endLead2, endLeadExtension1, endLeadExtension2])
      newRect = {
        left: innerRect.left - arrowLength,
        top: innerRect.top - arrowLength,
        width: innerRect.width + 2 * arrowLength,
        height: innerRect.height + 2 * arrowLength
      }

    } else {
      const innerRect = this.calculate(start, end)
      newRect = {
        left: innerRect.left - arrowLength,
        top: innerRect.top - arrowLength,
        width: innerRect.width + 2 * arrowLength,
        height: innerRect.height + 2 * arrowLength
      }
    }
    this.rect = newRect
    const rect = this.rect

    this.svgElement.setAttribute("width", rect.width);
    this.svgElement.setAttribute("height", rect.height);
    this.svgElement.setAttribute("viewBox", `${rect.left} ${rect.top} ${rect.width} ${rect.height}`)

    const line = this.svgElement.querySelector('[mainLine]')
    if (line) {
      line.setAttribute("x1", start.x)
      line.setAttribute("y1", start.y)
      line.setAttribute("x2", end.x)
      line.setAttribute("y2", end.y)
    }

    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'

    const margin = this.annotation.measure ? 0 : MARGIN_DISTANCE
    this.outerLine.setAttribute("height", rect.height);
    this.outerLine.setAttribute("viewBox", `${rect.left} ${rect.top} ${rect.width + margin * 2} ${rect.height + margin * 2}`)
    this.outerLine.style.left = `${rect.left - margin}px`
    this.outerLine.style.top = `${rect.top - margin}px`
    this.outerLine.style.width = `${rect.width + margin * 2}px`
    this.outerLine.style.height = `${rect.height + margin * 2}px`

    if (this.deletetButton) {
      this.deletetButton.style.left = (rect.left + rect.width + MARGIN_DISTANCE - 30) + 'px'
      this.deletetButton.style.top = (rect.top - MARGIN_DISTANCE - 30) + 'px'
    }
    if (this.replyButton) {
      this.replyButton.style.left = (rect.left + rect.width + MARGIN_DISTANCE - 65) + 'px'
      this.replyButton.style.top = (rect.top - MARGIN_DISTANCE - 30) + 'px'
    }
    
    if (this.annotation.measure) {
      const direction = this.annotation.leadLength > this.annotation.leadExtension
      this.startCircle.setAttribute("cx", direction ? endLead1.x : endLeadExtension1.x)
      this.startCircle.setAttribute("cy", direction ? endLead1.y : endLeadExtension1.y)
      this.endCircle.setAttribute("cx", direction ? endLead2.x : endLeadExtension2.x)
      this.endCircle.setAttribute("cy", direction ? endLead2.y : endLeadExtension2.y)
    } else {
      this.startCircle.setAttribute("cx", start.x + margin)
      this.startCircle.setAttribute("cy", start.y + margin)
      this.endCircle.setAttribute("cx", end.x + margin)
      this.endCircle.setAttribute("cy", end.y + margin)
    }
    
    this.moveLine.setAttribute("x1", start.x + margin)
    this.moveLine.setAttribute("y1", start.y + margin)
    this.moveLine.setAttribute("x2", end.x + margin)
    this.moveLine.setAttribute("y2", end.y + margin)

    if (this.annotation.head || this.annotation.tail) {
      let arrow = new ArrowHelper(start, end)
      const angle = this.annotation.measure ? 45 : 30
      const actualbdwidth = (this.annotation.borderWidth || 2) * scale
      const arrowLength = Math.max(actualbdwidth * 3, (this.annotation.measure ? 10 : 12) * scale)

      if (this.annotation.head && this.annotation.head === 'OpenArrow') {
        let head = this.svgElement.querySelector('[head]')
        if (head) {
          let arrowPoints = arrow.getStartArrowPoints(angle, arrowLength)
          let points = ''
          arrowPoints && arrowPoints.forEach(element => {
            if (points.length > 0) {
              points = points + ' '
            }
            points = points + `${element.x},${element.y}`
          })
          if (points.length > 0) {
            head.setAttribute('points', points)
          }
        }
      }
      if (this.annotation.tail && this.annotation.tail === 'OpenArrow') {
        let tail = this.svgElement.querySelector('[tail]')
        if (tail) {
          let arrowPoints = arrow.getEndArrowPoints(angle, arrowLength)
          let points = ''
          arrowPoints && arrowPoints.forEach(element => {
            if (points.length > 0) {
              points = points + ' '
            }
            points = points + `${element.x},${element.y}`
          })
          if (points.length > 0) {
            tail.setAttribute('points', points)
          }
        }
      }
    }

    this.annotation.measure && isUpdateMeasure && this.updateMeasure(start, end)
  }

  async updateMeasure (start, end) {
    // 更新测量数据
    if (this.startState && this.startState.operator !== 'move') {
      const initPoint = this.getInitialPoint()
      const rect = {
        left: Math.min(initPoint.start.x, initPoint.end.x),
        top: Math.min(initPoint.start.y, initPoint.end.y),
        right: Math.max(initPoint.start.x, initPoint.end.x),
        bottom: Math.max(initPoint.start.y, initPoint.end.y)
      }
      const linePoints = [initPoint.start.x, initPoint.start.y, initPoint.end.x, initPoint.end.y]

      const annotationData = {
        pagePtr: this.annotation.pagePtr,
        annotPtr: this.annotation.annotPtr,
        rulerPtr: this.annotation.rulerPtr,
        pageIndex: this.annotation.pageIndex,
        rect,
        linePoints,
      }
      if (this.annotation.contents) {
        annotationData.contents = this.annotation.contents
        annotationData.labelPosition = this.annotation.labelPosition
      }
      this.annotation.rect = rect
      this.annotation.linePoints = linePoints

      await this.layer.documentViewer.annotationManager.editMeasurementAnnotation(annotationData)
      await this.getMeasureData()
    }

    if (this.startState && this.startState.operator === 'middle') {
      const annotationData = {
        pagePtr: this.annotation.pagePtr,
        annotPtr: this.annotation.annotPtr,
        rulerPtr: this.annotation.rulerPtr,
        pageIndex: this.annotation.pageIndex,
        leadLength: this.annotation.leadLength,
        leadExtension: this.annotation.leadExtension,
        leadOffset: 1 - this.annotation.leadLength,
        contents: this.annotation.contents
      }
      this.annotation.leadOffset = 1 - this.annotation.leadLength
      await this.layer.documentViewer.annotationManager.editMeasurementAnnotation(annotationData)
    }

    // 更新绘制的元素
    const startLine = this.svgElement.querySelector('[startLine]')
    const endLine = this.svgElement.querySelector('[endLine]')
    const startLead = this.svgElement.querySelector('[startLead]')
    const endLead = this.svgElement.querySelector('[endLead]')
    const startLeadExtension = this.svgElement.querySelector('[startLeadExtension]')
    const endLeadExtension = this.svgElement.querySelector('[endLeadExtension]')
    
    const midpoint = {
      x: (start.x + end.x) / 2,
      y: (start.y + end.y) / 2
    }
    const leadLength = this.annotation.leadLength * this.layer.viewport.scale
    const leadExtension = this.annotation.leadExtension * this.layer.viewport.scale
    const dx = end.x - start.x
    const dy = end.y - start.y
    const length = Math.sqrt(dx * dx + dy * dy)
    const unitX = dx / length
    const unitY = dy / length
    const perpX = -unitY
    const perpY = unitX

    if (startLine) {
      const line1End = {
        x: start.x + unitX * (length / 2 - this.textGap / 2),
        y: start.y + unitY * (length / 2 - this.textGap / 2),
      }
      startLine.setAttribute("x1", start.x)
      startLine.setAttribute("y1", start.y)
      startLine.setAttribute("x2", line1End.x)
      startLine.setAttribute("y2", line1End.y)
    }
    if (endLine) {
      const line2Start = {
        x: start.x + unitX * (length / 2 + this.textGap / 2),
        y: start.y + unitY * (length / 2 + this.textGap / 2),
      }
      endLine.setAttribute("x1", line2Start.x)
      endLine.setAttribute("y1", line2Start.y)
      endLine.setAttribute("x2", end.x)
      endLine.setAttribute("y2", end.y)
    }
    if (startLead) {
      const endLine1 = {
        x: start.x + perpX * leadLength,
        y: start.y + perpY * leadLength
      }
      startLead.setAttribute("x1", start.x)
      startLead.setAttribute("y1", start.y)
      startLead.setAttribute("x2", endLine1.x)
      startLead.setAttribute("y2", endLine1.y)
    }
    if (endLead) {
      const endLine2 = {
        x: end.x + perpX * leadLength,
        y: end.y + perpY * leadLength
      }
      endLead.setAttribute("x1", end.x)
      endLead.setAttribute("y1", end.y)
      endLead.setAttribute("x2", endLine2.x)
      endLead.setAttribute("y2", endLine2.y)
    }
    if (startLeadExtension) {
      const reverseEndLine1 = {
        x: start.x - perpX * leadExtension,
        y: start.y - perpY * leadExtension
      }
      startLeadExtension.setAttribute("x1", start.x)
      startLeadExtension.setAttribute("y1", start.y)
      startLeadExtension.setAttribute("x2", reverseEndLine1.x)
      startLeadExtension.setAttribute("y2", reverseEndLine1.y)
    }
    if (endLeadExtension) {
      const reverseEndLine2 = {
        x: end.x - perpX * leadExtension,
        y: end.y - perpY * leadExtension
      }
      endLeadExtension.setAttribute("x1", end.x)
      endLeadExtension.setAttribute("y1", end.y)
      endLeadExtension.setAttribute("x2", reverseEndLine2.x)
      endLeadExtension.setAttribute("y2", reverseEndLine2.y)
    }

    if (this.textEle) {
      const angleDeg = Math.atan2(end.y - start.y, end.x - start.x) / Math.PI * 180
      const rotateAngle = Math.abs(angleDeg) > 90 ? angleDeg - 180 : angleDeg
      const angle = Math.abs(angleDeg) > 90 ? 180 - Math.abs(angleDeg) : angleDeg
      const formatAngle = Math.abs(angle).toFixed(this.decimalPlaces) * 1
      this.annotation.angle = formatAngle

      this.textEle.setAttribute('x', midpoint.x)
      this.textEle.setAttribute('y', midpoint.y)
      this.textEle.setAttribute('transform', `rotate(${rotateAngle} ${midpoint.x} ${midpoint.y})`)
    }

    this.middleCircle.setAttribute("cx", midpoint.x)
    this.middleCircle.setAttribute("cy", midpoint.y)

    if (this.startState && this.startState.operator === 'move') return

    if (this.textEle) {
      this.textEle.textContent = this.annotation.contents
    }

    // 更新看板数据
    const X = (Math.abs(initPoint.end.x - initPoint.start.x) * this.annotation.factor).toFixed(this.decimalPlaces)
    const Y = (Math.abs(initPoint.end.y - initPoint.start.y) * this.annotation.factor).toFixed(this.decimalPlaces)

    this.annotation.X = X
    this.annotation.Y = Y

    this.layer.eventBus.dispatch('updateMeasureData', {
      scale: this.annotation.scale,
      precision: this.annotation.precision,
      length: this.annotation.measureResult,
      angle: this.annotation.angle,
      X,
      Y,
    })
  }

  async handlePropertyPanel(props) {
    const annotation = this.annotation
    if (this.layer.annotationStore.selectedElementName !== annotation.name) return

    let updatePropsNum = 0

    for (const key in props) {
      updatePropsNum += await this.setCustomData(key, props[key], true)
    }

    updatePropsNum > 0 && this.sendData(props)
  }

  async setCustomData(key, value, noSend = false) {
    const annotation = this.annotation
    let isUpdate = 0

    for (const item in annotation) {
      if (item === key && this.annotation[item] !== value) {
        annotation[item] = value
        isUpdate = 1

        // Styles
        if (key === 'borderColor') {
          const lineNames = ['startLine', 'endLine', 'startLead', 'endLead', 'startLeadExtension', 'endLeadExtension', 'head', 'tail']
          lineNames.forEach(name => {
            const line = this.svgElement.querySelector(`[${name}]`)
            line && line.setAttribute('stroke', value)
          })
        }
        if (key === 'opacity') {
          this.svgElement.style.opacity = value
        }
        if (key === 'borderWidth') {
          const lineNames = ['startLine', 'endLine', 'startLead', 'endLead', 'startLeadExtension', 'endLeadExtension', 'head', 'tail']
          lineNames.forEach(name => {
            const line = this.svgElement.querySelector(`[${name}]`)
            line && line.setAttribute('stroke-width', value)
          })
          this.update({ start: this.start, end: this.end }, false)
          this.showPopup()
        }
        if (key === 'borderStyle') {
          const lineNames = ['startLine', 'endLine']
          lineNames.forEach(name => {
            const line = this.svgElement.querySelector(`[${name}]`)
            if (line && value === 'solid') {
              line.setAttribute('stroke-dasharray', '')
            }
            if (line && value === 'dashed') {
              let dashString = ''
              const scaleValue = Math.ceil(this.ratio * this.layer.viewport.scale)

              if (annotation.dashes.length === 1 || (annotation.dashes.length === 2 && annotation.dashes[0] === annotation.dashes[1])) {
                dashString = Math.ceil(annotation.dashes[0] * scaleValue) + ''
              } else if (annotation.dashes.length > 2) {
                dashString = annotation.dashes.map(dash => Math.ceil(dash * scaleValue)).join(', ')
              }
              line.setAttribute('stroke-dasharray', dashString)
            }
          })
        }
        if (key === 'dashes') {
          const lineNames = ['startLine', 'endLine']
          lineNames.forEach(name => {
            const line = this.svgElement.querySelector(`[${name}]`)
            line && line.setAttribute('stroke-dasharray', Math.ceil(value[0] * this.ratio * this.layer.viewport.scale))
          })
        }
        if (key === 'head') {
          const head = this.svgElement.querySelector('[head]')
          if (head) {
            head.style.display = value ? '' : 'none'
          } else if (value) {
            const actualbdwidth = (annotation.borderWidth || 2) * this.layer.viewport.scale
            const arrowLength = Math.max(actualbdwidth * 3, (annotation.measure ? 10 : 12) * this.layer.viewport.scale)
            
            let arrowBuild = new ArrowHelper(this.start, this.end)
            let arrowHead = arrowBuild.getStartArrow(45, arrowLength)
            if (arrowHead) {
              arrowHead.setAttribute('head','')
              arrowHead.setAttribute("stroke", annotation.borderColor)
              arrowHead.setAttribute("stroke-width", actualbdwidth)
              arrowHead.setAttribute("fill", "none")
              this.svgElement.append(arrowHead)
            }
          }
        }
        if (key === 'tail') {
          const tail = this.svgElement.querySelector('[tail]')
          if (tail) {
            tail.style.display = value ? '' : 'none'
          } else if (value) {
            const actualbdwidth = (annotation.borderWidth || 2) * this.layer.viewport.scale
            const arrowLength = Math.max(actualbdwidth * 3, (annotation.measure ? 10 : 12) * this.layer.viewport.scale)

            let arrowBuild = new ArrowHelper(this.start, this.end)
            let arrowTail = arrowBuild.getEndArrow(45, arrowLength)
            if (arrowTail) {
              arrowTail.setAttribute('tail','')
              arrowTail.setAttribute("stroke", annotation.borderColor)
              arrowTail.setAttribute("stroke-width", actualbdwidth)
              arrowTail.setAttribute("fill", "none")
              this.svgElement.append(arrowTail)
            }
          }
        }
        if (key === 'color') {
          this.textEle.setAttribute('fill', value)
        }
        if (key === 'fontSize') {
          this.textEle.setAttribute('font-size', value)
        }
        if (key === 'fontName') {
          this.textEle.style.fontFamily = value
        }
        if (key === 'fontStyle') {
          const fontStyle = value
          if (fontStyle === 'boldItalic') {
            this.textEle.style.fontStyle = 'italic'
            this.textEle.style.fontWeight = 'bold'
            annotation.isBold = true
            annotation.isItalic = true
          } else if (fontStyle === 'bold') {
            this.textEle.style.fontStyle = 'normal'
            this.textEle.style.fontWeight = 'bold'
            annotation.isBold = true
            annotation.isItalic = false
          } else if (fontStyle === 'italic') {
            this.textEle.style.fontStyle = 'italic'
            this.textEle.style.fontWeight = 'normal'
            annotation.isBold = false
            annotation.isItalic = true
          } else {
            annotation.isItalic = false
            annotation.isBold = false
            this.textEle.style.fontStyle = 'normal'
            this.textEle.style.fontWeight = 'normal'
          }
        }
        if (key === 'contents') {
          this.textEle.textContent = value
        }

        // Measure Properties
        if (key === 'scale' || key === 'precision') {
          if (key === 'precision') {
            this.decimalPlaces = Math.abs(annotation.precision).toString().split('.')[1]?.length || 0
          }

          await this.layer.documentViewer.annotationManager.editMeasurementAnnotation({
            rulerPtr: annotation.rulerPtr,
            [key]: value
          })
          await this.getMeasureData()
          await this.layer.documentViewer.annotationManager.editMeasurementAnnotation({
            annotPtr: annotation.annotPtr,
            contents: annotation.contents
          })
          
          const factor = await this.layer.documentViewer.messageHandler.sendWithPromise('GetFactor', annotation.rulerPtr)
          const initPoint = this.getInitialPoint()
          const X = (Math.abs(initPoint.end.x - initPoint.start.x) * factor).toFixed(this.decimalPlaces)
          const Y = (Math.abs(initPoint.end.y - initPoint.start.y) * factor).toFixed(this.decimalPlaces)

          annotation.factor = factor
          annotation.X = X
          annotation.Y = Y

          if (this.textEle) {
            this.textEle.textContent = annotation.contents
          }
          if (this.layer.annotationStore.selectedElementName === this.annotation.name && !document.querySelector('.measure-panel').classList.contains('closed')) {
            this.layer.eventBus.dispatch('measurePropertyChange', this.annotation)
          }

          this.layer.eventBus.dispatch('updateMeasureData', {
            scale: value,
            length: annotation.measureResult,
            X,
            Y,
          })

          isUpdate = 0
        }

        if (['fontSize', 'scale', 'precision', 'contents'].includes(key)) {
          const startLine = this.svgElement.querySelector('[startLine]')
          const endLine = this.svgElement.querySelector('[endLine]')
          const dx = this.end.x - this.start.x
          const dy = this.end.y - this.start.y
          const length = Math.sqrt(dx * dx + dy * dy)
          const unitX = dx / length
          const unitY = dy / length
          const textGap = this.getTextWidth()

          if (startLine) {
            const line1End = {
              x: this.start.x + unitX * (length / 2 - textGap / 2),
              y: this.start.y + unitY * (length / 2 - textGap / 2),
            }
            startLine.setAttribute("x1", this.start.x)
            startLine.setAttribute("y1", this.start.y)
            startLine.setAttribute("x2", line1End.x)
            startLine.setAttribute("y2", line1End.y)
          }
          if (endLine) {
            const line2Start = {
              x: this.start.x + unitX * (length / 2 + textGap / 2),
              y: this.start.y + unitY * (length / 2 + textGap / 2),
            }
            endLine.setAttribute("x1", line2Start.x)
            endLine.setAttribute("y1", line2Start.y)
            endLine.setAttribute("x2", this.end.x)
            endLine.setAttribute("y2", this.end.y)
          }
        }
      }
    }
    !noSend && this.sendData({ [key]: value })
    return isUpdate
  }

  sendData(props) {
    const annotation = this.annotation
    const annotationData = {
      pagePtr: annotation.pagePtr,
      annotPtr: annotation.annotPtr,
      rulerPtr: annotation.rulerPtr,
      pageIndex: annotation.pageIndex,
      contents: annotation.contents
    }

    const isFont = ['fontSize', 'fontName', 'color', 'fontStyle'].some(item => props[item])
    if (isFont) {
      annotationData.fontSize = annotation.fontSize
      annotationData.color = annotation.color
      const fontArr = {
        Courier: {
          normal: 'Courier',
          bold: 'Courier-Bold',
          italic: 'Courier-Oblique',
          boldItalic: 'Courier-BoldOblique'
        },
        Helvetica: {
          normal: 'Helvetica',
          bold: 'Helvetica-Bold',
          italic: 'Helvetica-Oblique',
          boldItalic: 'Helvetica-BoldOblique'
        },
        Times: {
          normal: 'Times',
          bold: 'Times-Bold',
          italic: 'Times-Italic',
          boldItalic: 'Times-BoldItalic'
        }
      }

      const fontName = annotation.fontName
      if (annotation.isBold && annotation.isItalic) {
        annotationData.fontName = fontArr[fontName].boldItalic
      } else if (annotation.isBold) {
        annotationData.fontName = fontArr[fontName].bold
      } else if (annotation.isItalic) {
        annotationData.fontName = fontArr[fontName].italic
      } else {
        annotationData.fontName = fontArr[fontName] ? fontArr[fontName].normal : fontName
      }
      delete props.fontName
    }
    if (props.hasOwnProperty('head') || props.hasOwnProperty('tail')) {
      annotationData.head = annotation.head
      annotationData.tail = annotation.tail
      delete props.head
      delete props.tail
    }
    if (props.dashes) {
      annotationData.borderStyle = annotation.borderStyle
      annotationData.borderWidth = annotation.borderWidth
    }
    if (props.borderStyle) {
      annotationData.dashes = props.borderStyle === 'solid' ? [] : annotation.dashes
      annotationData.borderWidth = annotation.borderWidth
    }

    const updateAnnot = Object.assign({}, annotationData, props)
    this.layer.documentViewer.annotationManager.editMeasurementAnnotation(updateAnnot)
  }

  async getMeasureData() {
    const { annotPtr, rulerPtr } = this.annotation
    const value = await this.layer.documentViewer.messageHandler.sendWithPromise('GetMeasurementResult', { annotPtr, rulerPtr })
    const formatValue = this.formatValue(value)

    this.annotation.measureResult = formatValue
    this.annotation.contents = formatValue + this.annotation.scale.translateUnit
  }

  showPopup() {
    const pageRect = this.layer.div.getBoundingClientRect()

    const left = pageRect.left + this.rect.left
    const top = this.rect.top + pageRect.top
    const right = left + this.rect.width
    const bottom = top + this.rect.height
    const rect = {
      left,
      top,
      right,
      bottom
    }
    this.layer.eventBus.dispatch('showMeasurementPopup', { rect })
    this.isPopupShow = true
  }

  hidePopup() {
    if (!this.isPopupShow) return
    this.isPopupShow = false
    this.layer.eventBus.dispatch('showMeasurementPopup')
  }

  async handlePopup(data) {
    if (data === 'delete') {
      this.handleDelete()
    }
    if (data === 'reply') {
      this.openReply()
    }
  }

  // 根据位数保留小数
  formatValue(value) {
    return value % 1 === 0 ? value : Number(value.toFixed(this.decimalPlaces))
  }

  // 获取文本宽度
  getTextWidth() {
    const canvas = document.createElement('canvas')
    const context = canvas.getContext('2d')

    context.font = `${this.annotation.fontSize}px ${this.annotation.fontFamily}`
    const metrics = context.measureText(this.annotation.contents)
    const gap = metrics.width * 1.1

    const dx = this.end.x - this.start.x
    const dy = this.end.y - this.start.y
    const length = Math.sqrt(dx * dx + dy * dy)

    this.textGap = length > gap ? gap : length

    return this.textGap
  }

  async handleKeyDown (e) {
    if (e.target.tagName === 'INPUT' || e.target.tagName === 'TEXTAREA') return
    
    let keyCode = e.keyCode || e.which
    if (keyCode === 8 || keyCode === 46) { // 8 backspace键，46 delete键
      this.handleDelete()
    }
  }
}
