import Base from './base';
import { getActualPoint, getClickPoint, createSvg, createElement } from './utils';
import { onClickOutside } from '../ui_utils'

export default class Shape extends Base {
  
  constructor ({
    annotation,
    layer,
    show = false,
    enableReply = true
  }) {
    super({
      annotation,
      layer,
    })
    this.enableReply = enableReply

    this.hidden = true
    this.initial = false

    this.start = null
    this.end = null

    this.newStart = null
    this.newEnd = null

    this.svgElement = null

    this.startCircle = null
    this.endCircle = null

    this.show = show

    this.onMousedown = this.handleMouseDown.bind(this)
    this.onMouseup = this.handleMouseUp.bind(this)
    this.onMousemove = this.handleMouseMove.bind(this)
    this.onDelete = this.handleDelete.bind(this)
    this.onOpenReply = this.openReply.bind(this)
    this.render()
  }

  render () {
    const annotation = this.annotation

    const { start, end } = this.getActualRect(this.layer.viewport.scale)
    this.start = start
    this.end = end

    const rect = this.calculate(start, end)
    const actualbdwidth = (annotation.borderWidth || 2) * this.layer.viewport.scale

    if (!annotation.borderColor) {
      annotation.borderColor = annotation.color || '#000000'
    }
    
    const annotationContainer = document.createElement('div')
    annotationContainer.id = annotation.name
    annotationContainer.className = 'annotation'
    annotationContainer.style.top = rect.top + 'px'
    annotationContainer.style.left = rect.left + 'px'
    annotationContainer.style.width = rect.width + 'px'
    annotationContainer.style.height = rect.height + 'px'
    this.annotationContainer = annotationContainer

    this.svgElement = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    this.svgElement.setAttribute("viewBox", `0 0 ${rect.width} ${rect.height}`)
    this.svgElement.addEventListener('mousedown', this.handleClick.bind(this))

    let shapeElement
    if (annotation.type === 'square') {
      shapeElement = createSvg(
        "rect",
        {
          x: actualbdwidth / 2,
          y: actualbdwidth / 2,
          width: `${Math.abs(rect.width - actualbdwidth)}px`,
          height: `${Math.abs(rect.height - actualbdwidth)}px`,
          stroke: annotation.borderColor,
          'stroke-width': actualbdwidth,
          'stroke-opacity': annotation.opacity || 1,
          fill: annotation.fillColor ? annotation.fillColor : 'none',
          'fill-opacity': annotation.opacity || 1,
        }
      );
    } else {
      shapeElement = createSvg(
        "ellipse",
        {
          cx: rect.width / 2,
          cy: rect.height / 2,
          rx: (rect.width - actualbdwidth) / 2,
          ry: (rect.height - actualbdwidth) / 2,
          stroke: annotation.borderColor,
          'stroke-width': actualbdwidth,
          'stroke-opacity': annotation.opacity || 1,
          fill: annotation.fillColor ? annotation.fillColor : 'none',
          'fill-opacity': annotation.opacity || 1,
        }
      );
    }

    this.shapeElement = shapeElement

    this.svgElement.append(this.shapeElement)

    this.annotationContainer.append(this.svgElement)

    this.layer.div.append(this.annotationContainer)

    this.outerLineContainer = document.createElement('div')
    this.outerLineContainer.className = 'outline-container'
    this.deletetButton = createSvg(
      "svg",
      {
        width: "30",
        height: "30",
        viewBox: "0 0 30 30",
        fill: "none",
        class: 'delete-button'
      },
      {
        width: '30px',
        height: '30px'
      }
    );
    this.deletetButton.addEventListener('click', this.onDelete)
    this.deletetButton.innerHTML = this.deleteSvgStr

    this.deletetButton.style.left = (rect.left + rect.width - 30) + 'px'
    this.deletetButton.style.top = (rect.top + rect.height + 8) + 'px'

    this.replyButton = createElement(
      'div',
      {
        width: '30px',
        height: '30px',
        background: '#DDE9FF',
        borderRadius: '2px',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center'
      },
      {
        class: 'reply-button'
      }
    );
    this.replyButton.addEventListener('click', this.onOpenReply)
    this.replyButton.innerHTML = this.replySvgStr
    this.replyButton.style.left = (rect.left + rect.width - 65) + 'px'
    this.replyButton.style.top = (rect.top + rect.height + 8) + 'px'
    
    const outerLine = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    outerLine.style.position = 'absolute'
    outerLine.style.zIndex = 1
    outerLine.style.left = `${rect.left - 7}px`
    outerLine.style.top = `${rect.top - 7}px`
    outerLine.style.width = `${rect.width + 7 * 2}px`
    outerLine.style.height = `${rect.height + 7 * 2}px`
    this.outerLine = outerLine
    
    this.moveRect = createSvg(
      "rect",
      {
        class: "move",
        'data-id': "move",
        stroke: "#1460F3",
        'stroke-width': 1,
        'fill-opacity': 0,
        width: rect.width + 8,
        height: rect.height + 8,
        x: 3,
        y: 3
      }
    );
    
    this.topLeftRect = createSvg(
      "rect",
      {
        class: "nw-resize",
        'data-id': "nw-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: 0,
        y: 0
      }
    );

    this.LeftRect = createSvg(
      "rect",
      {
        class: "w-resize",
        'data-id': "w-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: 0,
        y: rect.height / 2 + 4
      }
    );

    this.bottomLeftRect = createSvg(
      "rect",
      {
        class: "sw-resize",
        'data-id': "sw-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: 0,
        y: rect.height + 8
      }
    );

    this.bottomRect = createSvg(
      "rect",
      {
        class: "s-resize",
        'data-id': "s-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width / 2 + 4,
        y: rect.height + 8
      }
    );
    
    this.bottomRightRect = createSvg(
      "rect",
      {
        class: "se-resize",
        'data-id': "se-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width + 8,
        y: rect.height + 8
      }
    );

    this.rightRect = createSvg(
      "rect",
      {
        class: "e-resize",
        'data-id': "e-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width + 8,
        y: rect.height / 2 + 4
      }
    );

    this.topRightRect = createSvg(
      "rect",
      {
        class: "ne-resize",
        'data-id': "ne-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width + 8,
        y: 0
      }
    );

    this.topRect = createSvg(
      "rect",
      {
        class: "n-resize",
        'data-id': "n-resize",
        fill: "#1460F3",
        stroke: "none",
        width: 6,
        height: 6,
        x: rect.width / 2 + 4,
        y: 0
      }
    );

    this.outerLine.append(this.moveRect)
    this.outerLine.append(this.topLeftRect)
    this.outerLine.append(this.LeftRect)
    this.outerLine.append(this.bottomLeftRect)
    this.outerLine.append(this.bottomRect)
    this.outerLine.append(this.bottomRightRect)
    this.outerLine.append(this.rightRect)
    this.outerLine.append(this.topRightRect)
    this.outerLine.append(this.topRect)
    this.outerLineContainer.append(this.outerLine)
    const isAnnotationPopup = this.layer.documentViewer.isAnnotationPopup
    isAnnotationPopup && this.outerLineContainer.append(this.deletetButton);
    (isAnnotationPopup && this.enableReply) && this.outerLineContainer.append(this.replyButton)
    this.initial = true
    if (this.show) {
      !this.layer.addMode && this.selectAnnotation()
      this.show = false
    }
  }

  getActualRect (s) {
    const annotation = this.annotation
    const { left: x1, top: y1, right: x2, bottom: y2 } = annotation.rect
    
    const start = getActualPoint(
      {
        x: x1,
        y: y1
      },
      s
    )
    const end = getActualPoint(
      {
        x: x2,
        y: y2
      },
      s
    )
    return {
      start,
      end
    }
  }

  rectCalc ({ start, end }) {
    return {
      top: start.y < end.y ? start.y : end.y,
      left: start.x < end.x ? start.x : end.x,
      width: Math.abs(end.x - start.x),
      height: Math.abs(end.y - start.y),
    };
  };

  calculate (start, end) {
    const initRect = this.rectCalc({ start, end });
    const actualbdwidth = (this.annotation.borderWidth || 2) * this.layer.viewport.scale

    return {
      top: initRect.top - actualbdwidth,
      left: initRect.left - actualbdwidth,
      width: initRect.width + actualbdwidth * 2,
      height: initRect.height + actualbdwidth * 2,
    }
  }

  getInitialPoint () {
    const s = this.layer.viewport.scale
    const startX = this.start.x
    const startY = this.start.y
    const endX = this.end.x
    const endY = this.end.y

    let x1, y1, x2, y2;
    x1 = startX / s;
    y1 = startY / s; 
    x2 = endX / s;
    y2 = endY / s;
    return {
      start: {
        x: x1,
        y: y1,
      },
      end: {
        x: x2,
        y: y2,
      },
    };
  }

  selectAnnotation () {
    if (this.layer.selectedElementName === this.annotation.name) return
    this.handleClick()
  }

  updateTool () {
    if (!this.initial) return
    if (this.hidden) {
      if (this.layer.selectedElementName === this.annotation.name) {
        this.layer.selectedElementName = null
      }
      this.outerLineContainer.remove()
    } else {
      if (this.layer.selectedElementName !== this.annotation.name) {
        const annotation = this.annotation
        this.layer.selectedElementName = annotation.name

        const annotationData = {
          ...annotation
        }
        delete annotationData.annotPtr
        delete annotationData.pagePtr
        delete annotationData.operate
        delete annotationData.targetPage

        this.layer.eventBus.dispatch('annotationSelected', annotationData)
      }
      this.layer.div.append(this.outerLineContainer)
      this.outerLine.addEventListener('mousedown', this.onMousedown)
      this.outerLine.addEventListener('touchstart', this.onMousedown)

      const documentViewer = this.layer.documentViewer
      const scrollElement = documentViewer.getScrollViewElement()
      scrollElement.style.touchAction = 'none'
    }
  }

  handleClick (event) {
    if (!this.hidden || this.layer.annotationStore.creating || ['redaction', 'remove'].includes(this.layer.tool) || document.querySelector('.compare-document-container').contains(event?.target)) return
    this.hidden = false
    this.updateTool()
    onClickOutside([this.svgElement, this.outerLine, this.deletetButton, this.replyButton, document.querySelector('.toggle-button')], this.handleOutside.bind(this))
  }

  handleOutside () {
    this.hidden = true
    if (this.layer.selectedElementName === this.annotation.name) {
      this.layer.selectedElementName = null
    }
    this.outerLine?.removeEventListener('mousedown', this.onMousedown)
    this.outerLine?.removeEventListener('touchstart', this.onMousedown)
    this.outerLineContainer?.remove()

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    scrollElement.style.touchAction = 'auto'
  }

  handleMouseDown (event) {
    if (event.button !== 0 && event.type === 'mousedown') return
    if (this.layer.tool) {
      event.stopPropagation()
    }
    const operatorId = event.target.getAttribute('data-id')
  
    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    this.scrollTop = scrollElement.scrollTop
    this.scrollLeft = scrollElement.scrollLeft

    const { pageX, pageY } = getClickPoint(event)
    this.startState = {
      operator: operatorId,
      clickX: pageX,
      clickY: pageY,
    }

    document.addEventListener('mousemove', this.onMousemove)
    document.addEventListener('mouseup', this.onMouseup)
    document.addEventListener('touchmove', this.onMousemove)
    document.addEventListener('touchend', this.onMouseup)
  };

  handleMouseMove (event) {
    if (event.button !== 0 && event.type === 'mousemove') return
    if (event.type === 'touchmove') {
      document.querySelector('.document-container').addEventListener('scroll', event => event.preventDefault())
    }
    this.moving = true
    let { pageX, pageY } = getClickPoint(event)

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    const scrollTop = scrollElement.scrollTop
    const scrollLeft = scrollElement.scrollLeft
    pageX = pageX + scrollLeft - this.scrollLeft
    pageY = pageY + scrollTop - this.scrollTop

    const start = this.start
    const end = this.end
    const startState = this.startState

    const { width, height } = this.layer.viewport
    if (startState.operator === 'nw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(end.y - 5, y1)
      y1 = Math.max(0, y1)

      this.update({
        start: { x: x1, y: y1 },
        end,
      });
    } else if (startState.operator === 'w-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)

      this.update({
        start: { x: x1, y: start.y },
        end,
      });
    } else if (startState.operator === 'sw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.end.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(height, y1)
      y1 = Math.max(start.y + 5, y1)

      this.update({
        start: { x: x1, y: start.y },
        end: { x: end.x, y: y1 },
      });
    } else if (startState.operator === 's-resize') {
      let y1 = pageY - (startState.clickY - this.end.y);
      y1 = Math.min(height, y1)
      y1 = Math.max(start.y + 5, y1)
      this.update({
        start,
        end: { x: end.x, y: y1 },
      });
    } else if (startState.operator === 'se-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.end.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(height, y)
      y = Math.max(start.y + 5, y)
      this.update({
        start,
        end: { x, y },
      });
    } else if (startState.operator === 'e-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      this.update({
        start,
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'ne-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.start.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(end.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start: { x: start.x, y },
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'n-resize') {
      let y = pageY - (startState.clickY - this.start.y);
      y = Math.min(end.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start: { x: start.x, y },
        end,
      });
    } else if (startState.operator === 'move') {
      let x1, y1, x2, y2
      const documentViewer = this.layer.documentViewer

      let windowCoordinates = this.getMouseLocation(event);
      const page = documentViewer.getSelectedPage(windowCoordinates, windowCoordinates)
      if (page.first && page.first !== (this.layer.page + 1)) {
        const pagePoint = documentViewer.windowToPage(windowCoordinates, page.first, false)

        const annotationWidth = Math.abs(start.x - end.x)
        const annotationHeight = Math.abs(start.y - end.y)
        if (documentViewer.pdfViewer.spreadMode === 0 && documentViewer.pdfViewer.scrollMode === 0) {
          x1 = pageX - (startState.clickX - this.start.x)
          y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
          x2 = pageX - (startState.clickX - this.end.x)
          y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
        } else if (documentViewer.pdfViewer.spreadMode === 0 && documentViewer.pdfViewer.scrollMode === 1) {
          x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
          y1 = pageY - (startState.clickY - this.start.y)
          x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
          y2 = pageY - (startState.clickY - this.end.y)
        } else if (documentViewer.pdfViewer.spreadMode === 1 && documentViewer.pdfViewer.scrollMode === 0) {
          const minPage = Math.min(page.first, this.layer.page + 1)
          const maxPage = Math.max(page.first, this.layer.page + 1)
          if ((maxPage - minPage) > 2 || ((minPage + 1) === maxPage && minPage % 2 === 0)) {
            return
          }
          if ((minPage + 1) === maxPage) {
            x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
            y1 = pageY - (startState.clickY - this.start.y)
            x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
            y2 = pageY - (startState.clickY - this.end.y)
          } else if ((minPage + 2) === maxPage) {
            x1 = pageX - (startState.clickX - this.start.x)
            y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
            x2 = pageX - (startState.clickX - this.end.x)
            y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
          }
        } else if (documentViewer.pdfViewer.spreadMode === 2 && documentViewer.pdfViewer.scrollMode === 0) {
          const minPage = Math.min(page.first, this.layer.page + 1)
          const maxPage = Math.max(page.first, this.layer.page + 1)
          if ((maxPage - minPage) > 2 || ((minPage + 1) === maxPage && !(minPage === 1 && maxPage === 2) && maxPage % 2 === 0 || (minPage === 1 && maxPage === 3))) {
            return
          }
          if ((minPage + 2) === maxPage || (minPage === 1 && maxPage === 2)) {
            x1 = pageX - (startState.clickX - this.start.x)
            y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
            x2 = pageX - (startState.clickX - this.end.x)
            y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
          } else if ((minPage + 1) === maxPage) {
            x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
            y1 = pageY - (startState.clickY - this.start.y)
            x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
            y2 = pageY - (startState.clickY - this.end.y)
          }
        }
        this.layer = documentViewer.pdfViewer._pages[page.first - 1].compdfAnnotationLayer

        this.annotationContainer.remove()
        this.outerLineContainer.remove()
        const container = this.layer.div
        container.append(this.annotationContainer)
        container.append(this.outerLineContainer)
      } else {
        y1 = pageY - (startState.clickY - this.start.y);
        x1 = pageX - (startState.clickX - this.start.x);
        x2 = pageX - (startState.clickX - this.end.x);
        y2 = pageY - (startState.clickY - this.end.y);
        const windowPoint1 = documentViewer.pageToWindow({
          x: x1,
          y: y1
        }, this.layer.page + 1, false)
        const windowPoint2 = documentViewer.pageToWindow({
          x: x2,
          y: y2
        }, this.layer.page + 1, false)
        let x = 0, y = 0
        if (pageY > startState.clickY) {
          y = scrollElement.offsetHeight - (windowPoint2.y - scrollElement.scrollTop - scrollElement.offsetTop)
          y = y < 100 ? 5 : 0
        } else if (pageY < startState.clickY) {
          y = windowPoint1.y - scrollElement.scrollTop - scrollElement.offsetTop
          y = y < 100 ? -5 : 0
        }
        if (pageX > startState.clickX) {
          x = scrollElement.offsetWidth - (windowPoint2.x - scrollElement.scrollLeft - scrollElement.offsetLeft)
          x = x < 100 ? 5 : 0
        } else if (pageX < startState.clickX) {
          x = (windowPoint1.x - scrollElement.scrollLeft - scrollElement.offsetLeft)
          x = x < 100 ? -5 : 0
        }

        if (x || y) {
          scrollElement.scrollTop += y
          scrollElement.scrollLeft += x
        }
        const rect = {
          width: Math.abs(start.x - end.x),
          height: Math.abs(start.y - end.y)
        }
        if (x1 < x2) {
          x1 = Math.max(0, x1)
          x1 = Math.min(width - rect.width, x1)
          x2 = Math.max(rect.width, x2)
          x2 = Math.min(width, x2)
        } else {
          x2 = Math.max(0, x2)
          x2 = Math.min(width - rect.width, x2)
          x1 = Math.max(rect.width, x1)
          x1 = Math.min(width, x1)
        }

        if (y1 < y2) {
          y1 = Math.max(0, y1)
          y1 = Math.min(height - rect.height, y1)
          y2 = Math.max(rect.height, y2)
          y2 = Math.min(height, y2)
        } else {
          y2 = Math.max(0, y2)
          y2 = Math.min(height - rect.height, y2)
          y1 = Math.max(rect.height, y1)
          y1 = Math.min(height, y1)
        }
      }

      this.update({
        start: { x: x1, y: y1 },
        end: { x: x2, y: y2 },
      });
    }
    startState.clickX = pageX
    startState.clickY = pageY
  }

  handleDelete (event) {
    if (this.layer.tool && event) {
      event.stopPropagation()
    }

    this.delete()
    const annotationData = {
      type: 'delete',
      annotation: {
        operate: "del-annot",
        name: this.annotation.name,
        pageIndex: this.layer.page
      }
    }
    this.annotation.annotPtr && (annotationData.annotation.annotPtr = this.annotation.annotPtr)

    this.layer.eventBus.dispatch('annotationChanged', annotationData)
  }

  delete () {
    this.handleOutside()
    this.annotationContainer?.remove()
    this.annotation.isDelete = true
    this.layer.deleteAnnotationInLayer({
      name: this.annotation.name
    })
  }

  handleMouseUp (event) {
    if (event.button !== 0 && event.type === 'mouseup') return
    if (this.layer.tool) {
      event.stopPropagation()
    }
    if (event.type === 'touchend') {
      document.querySelector('.document-container').style.overflow = 'auto'
    }
    this.moving = false
    document.removeEventListener('mousemove', this.onMousemove)
    document.removeEventListener('mouseup', this.onMouseup)
    document.removeEventListener('touchmove', this.onMousemove)
    document.removeEventListener('touchend', this.onMouseup)
    this.scrollLeft = 0
    this.scrollTop = 0

    const annotation = this.annotation
    const { start, end } = this.getInitialPoint()

    const rect = {
      left: start.x,
      top: start.y,
      right: end.x,
      bottom: end.y
    }

    if (rect === annotation.rect && annotation.pageIndex === this.layer.page) return

    annotation.rect = rect
    if (annotation.pageIndex !== this.layer.page) {
      const annotationData = {
        type: 'delete',
        annotation: {
          operate: "del-annot",
          name: annotation.name,
          pageIndex: annotation.pageIndex,
          annotPtr: annotation.annotPtr
        }
      }
      this.layer.eventBus.dispatch('annotationChanged', annotationData)

      const newAnnotationData = Object.assign({}, annotation)
      delete newAnnotationData.annotPtr
      delete newAnnotationData.pagePtr
      delete newAnnotationData.name
      newAnnotationData.operate = 'add-annot'
      newAnnotationData.pageIndex = this.layer.page
      this.layer.eventBus.dispatch('annotationChanged', {
        type: 'add',
        show: true,
        annotation: newAnnotationData
      })

      this.annotationContainer.remove()
      this.outerLineContainer.remove()
    } else {
      const annotationData = {
        operate: "mod-annot",
        name: annotation.name,
        pageIndex: this.layer.page,
        pagePtr: annotation.pagePtr,
        annotPtr: annotation.annotPtr,
        rect
      }

      !annotation.annotPtr && (annotationData.linePoints = [rect.left, rect.bottom, rect.right, rect.top])

      this.layer.eventBus.dispatch('annotationChanged', {
        type: 'modify',
        annotation: annotationData
      })
    }
  }

  update ({ start, end }) {
    const rect = this.calculate(start, end)

    this.start = start
    this.end = end

    const actualbdwidth = (this.annotation.borderWidth || 2) * this.layer.viewport.scale

    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'
    
    this.svgElement.setAttribute("viewBox", `0 0 ${rect.width} ${rect.height}`)

    if (this.annotation.type === 'square') {
      this.shapeElement.setAttribute("width", rect.width - actualbdwidth);
      this.shapeElement.setAttribute("height", rect.height - actualbdwidth);
    } else {
      this.shapeElement.setAttribute('cx', rect.width / 2,)
      this.shapeElement.setAttribute('cy', rect.height / 2,)
      this.shapeElement.setAttribute('rx', (rect.width - actualbdwidth) / 2,)
      this.shapeElement.setAttribute('ry', (rect.height - actualbdwidth) / 2,)
    }

    this.outerLine.style.left = `${rect.left - 7}px`
    this.outerLine.style.top = `${rect.top - 7}px`
    this.outerLine.style.width = `${rect.width + 7 * 2}px`
    this.outerLine.style.height = `${rect.height + 7 * 2}px`

    this.deletetButton.style.left = (rect.left + rect.width - 30) + 'px'
    this.deletetButton.style.top = (rect.top + rect.height + 8) + 'px'
    this.replyButton.style.left = (rect.left + rect.width - 65) + 'px'
    this.replyButton.style.top = (rect.top + rect.height + 8) + 'px'
    
    this.moveRect.setAttribute('width',  rect.width + 8)
    this.moveRect.setAttribute('height',  rect.height + 8)
    
    this.LeftRect.setAttribute("y", rect.height / 2 + 4)

    this.bottomLeftRect.setAttribute("y", rect.height + 8)

    this.bottomRect.setAttribute("x", rect.width / 2 + 4)
    this.bottomRect.setAttribute("y", rect.height + 8)



    this.bottomRightRect.setAttribute("x", rect.width + 8)
    this.bottomRightRect.setAttribute("y", rect.height + 8)


    this.rightRect.setAttribute("x", rect.width + 8,)
    this.rightRect.setAttribute("y", rect.height / 2 + 4)

    
    this.topRightRect.setAttribute("x", rect.width + 8)

    this.topRect.setAttribute("x", rect.width / 2 + 4)
  }

  openReply () {
    this.layer.eventBus.dispatch('openAnnotationReply', this.annotation)
  }
}
