export class ContentEditManager {
  constructor(documentViewer) {
    this.documentViewer = documentViewer
    this.editAreaInfoPtr = null
    this.selectedEditBox = null
  }

  async startContentEditMode() {
    // for (const pageView of this.documentViewer.pdfViewer._pages) {
    //   if (pageView.contentContainer) {
    //     await pageView.contentContainer.init()
    //   }
    // }
    this.documentViewer.setToolMode('editor')
  }

  async endContentEditMode() {
    // for (const pageView of this.documentViewer.pdfViewer._pages) {
    //   if (pageView.contentContainer) {
    //     pageView.contentContainer.destroy()
    //     pageView.contentContainer = null
    //   }
    // }
    // await this.documentViewer.updateTextPtr()
    this.documentViewer.setToolMode('view')
  }

  getSelectedEditBox() {
    return this.selectedEditBox
  }

  setSelectedEditBox(editBox) {
    this.selectedEditBox = editBox
  }

  getCopyArea() {
    return this.editAreaInfo
  }
  setCopyArea(editAreaInfo) {
    this.editAreaInfo = editAreaInfo
  }

  async pasteAreaInPage(pageNumber, editAreaInfo) {
    if (!editAreaInfo) {
      const copyArea = this.getCopyArea()
      if (!copyArea) return
      editAreaInfo = {
        rect: copyArea.rect,
        editAreaInfoPtr: copyArea.editAreaInfoPtr
      }
    }
    const documentViewer = this.documentViewer
    const pageIndex = pageNumber - 1
    const page = documentViewer.pdfViewer._pages[pageIndex]
    let contentContainer =  page.contentContainer
    let editPagePtr = contentContainer && contentContainer.editPagePtr
    if (!editPagePtr) {
      contentContainer = await page.getEditAnnotation()
      editPagePtr = contentContainer.editPagePtr
    }
    const editAreaPtr = await documentViewer.messageHandler.sendWithPromise('PasteArea', {
      pagePtr: documentViewer.pagesPtr[pageIndex].pagePtr,
      editPagePtr,
      pageIndex,
      editAreaInfoPtr: editAreaInfo.editAreaInfoPtr,
      rect: editAreaInfo.rect
    })
    const frameEditor = await contentContainer.addEditArea(editAreaPtr)
    contentContainer.handleOperateList({
      editPagePtr,
      editAreaIndex: frameEditor.editAreaIndex,
      editAreaPtr,
      pageIndex
    })
  }

  getSelectedText(pageNumber) {
    const documentViewer = this.documentViewer
    if (pageNumber && (pageNumber < 1 || pageNumber > documentViewer.pagesCount)) return ''
    const pages = documentViewer.pdfViewer._pages
    let selectedText = ''
    if (pageNumber) {
      selectedText += this.getPageSelectedText(pageNumber)
    } else {
      for (let i = 0; i < pages.length; i++) {
        const contentContainer = pages[i].contentContainer
        if (!contentContainer) continue
        selectedText += this.getPageSelectedText(i + 1)
      }
    }
    return selectedText
  }

  getPageSelectedText(pageNumber) {
    if (!pageNumber) return ''
    const pageIndex = pageNumber - 1
    const documentViewer = this.documentViewer
    const pages = documentViewer.pdfViewer._pages
    const contentContainer = pages[pageIndex].contentContainer
    if (!contentContainer) return ''
    const frameEditorList = contentContainer.frameEditorList
    let selectedText = ''
    for (let i = 0; i < frameEditorList.length; i++) {
      const frame = frameEditorList[i]
      if (frame.type === 'text' && frame.selectedCharRange) {
        selectedText += frame.selectedText
      }
    }
    return selectedText
  }

  async selectedEditBoxText() {
    const selectedEditBox = this.getSelectedEditBox()
    if (!selectedEditBox || selectedEditBox.type !== 'text') return
    selectedEditBox.selectedAllText && await selectedEditBox.selectedAllText()
  }

  async getEditBoxSelectedText() {
    const selectedEditBox = this.getSelectedEditBox()
    if (!selectedEditBox) return
    if (!selectedEditBox.selectedCharRange) return ''
    const text = await selectedEditBox.getText()
    return text
  }
}
