import Base from '../annotation/base';
import { getActualPoint, getClickPoint, createSvg, createElement } from '../annotation/utils';
import { onClickOutsideUp } from '../ui_utils'

export default class PushButton extends Base {
  constructor ({
    annotation,
    layer,
    show = false,
    highlight,
  }) {
    super({
      annotation,
      layer,
    })
    this.hidden = true
    this.initial = false

    this.start = null
    this.end = null

    this.newStart = null
    this.newEnd = null

    this.startCircle = null
    this.endCircle = null

    this.show = show
    this.highlight = highlight

    this.onMousedown = this.handleMouseDown.bind(this)
    this.onMouseup = this.handleMouseUp.bind(this)
    this.onMousemove = this.handleMouseMove.bind(this)
    this.onDelete = this.handleDelete.bind(this)

    this.handlePropertyPanel = this.handlePropertyPanel.bind(this)
    this.setProperty = this.setProperty.bind(this)
    this.setHighlight = this.setHighlight.bind(this)
    this.render()
  }

  async render () {
    this.layer.eventBus._on('setProperty', this.setProperty)
    this.layer.eventBus._on('setHighlightForm', this.setHighlight)

    const annotation = this.annotation

    if (!this.annotation.backgroundColor) {
      this.annotation.backgroundColor = '#1460F3'
    }
    if (annotation.isBold && annotation.isItalic) {
      annotation.fontStyle = 'boldItalic'
    } else if (annotation.isBold) {
      annotation.fontStyle = 'bold'
    } else if (annotation.isItalic) {
      annotation.fontStyle = 'italic'
    } else {
      annotation.fontStyle = 'normal'
    }

    const { start, end } = this.getActualRect(this.layer.viewport.scale)
    this.start = start
    this.end = end

    const rect = this.calculate(start, end)
    
    const annotationContainer = document.createElement('div')
    annotationContainer.id = annotation.name
    annotationContainer.className = 'annotation'
    annotationContainer.style.top = rect.top + 'px'
    annotationContainer.style.left = rect.left + 'px'
    annotationContainer.style.width = rect.width + 'px'
    annotationContainer.style.height = rect.height + 'px'
    annotationContainer.style.backgroundColor = this.highlight ? '#4982E54D' : ''
    annotationContainer.style.overflow = 'hidden'
    this.annotationContainer = annotationContainer
    this.annotationContainer.addEventListener('mousedown', this.handleClick.bind(this))

    let shapeElement
    shapeElement = createElement(
      'div',
      {
        left: 0,
        top: 0,
        width: `${Math.abs(rect.width)}px`,
        height: `${Math.abs(rect.height)}px`,
        position: 'relative'
      }
    )
    this.shapeElement = shapeElement

    this.getImage()
    this.annotationContainer.append(this.shapeElement)
    this.layer.div.append(this.annotationContainer)

    this.buttonContainer = createElement(
      'a',
      {
        position: 'absolute',
        left: 0,
        top: 0,
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
        width: '100%',
        height: '100%',
        outline: 'none',
        borderColor: annotation.borderColor,
        borderWidth: (annotation.borderWidth || 1) + 'px',
        borderStyle: annotation.borderStyle || 'solid',
        color: annotation.color,
        backgroundColor: annotation.backgroundColor,
        fontFamily: annotation.fontName,
        fontSize: annotation.fontSize || 12 + 'px',
        fontWeight: annotation.isBold ? 'bold' : 'normal',
        fontStyle: annotation.isItalic ? 'italic' : 'normal',
        overflow: 'hidden',
        textDecoration: 'none',
        opacity: 0
      }
    )
    this.buttonContainer.innerHTML = annotation.title
    this.shapeElement.append(this.buttonContainer)

    this.buttonContainer.addEventListener('blur', this.onBlur)
    if (this.annotation.url) {
      this.buttonContainer.setAttribute('href', this.annotation.url)
      this.buttonContainer.setAttribute('target', '_blank')
    }

    this.outerLineContainer = document.createElement('div')
    this.outerLineContainer.className = 'outline-container'
    this.deletetButton = createSvg(
      "svg",
      {
        width: "30",
        height: "30",
        viewBox: "0 0 30 30",
        fill: "none",
        class: 'delete-button'
      },
      {
        width: '30px',
        height: '30px'
      }
    );
    this.deletetButton.addEventListener('click', this.onDelete)
    this.deletetButton.innerHTML = this.deleteSvgStr

    const left = (rect.left + rect.width - 30) + 'px'
    const top = (rect.top + rect.height + 8) + 'px'
    this.deletetButton.style.left = left
    this.deletetButton.style.top = top
    
    const outerLine = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    outerLine.style.position = 'absolute'
    outerLine.style.zIndex = 1
    outerLine.style.left = `${rect.left - 8}px`
    outerLine.style.top = `${rect.top - 8}px`
    outerLine.style.width = `${rect.width + 8 * 2}px`
    outerLine.style.height = `${rect.height + 8 * 2}px`
    this.outerLine = outerLine
    
    this.moveRect = createSvg(
      "rect",
      {
        class: "move",
        'data-id': "move",
        stroke: "#4982E6",
        'stroke-width': 1,
        'fill-opacity': 0,
        width: rect.width + 8,
        height: rect.height + 8,
        x: 4,
        y: 4
      }
    );
    
    this.topLeftRect = createSvg(
      "rect",
      {
        class: "nw-resize",
        'data-id': "nw-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: 1,
        y: 1
      }
    );

    this.LeftRect = createSvg(
      "rect",
      {
        class: "w-resize",
        'data-id': "w-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: 1,
        y: rect.height / 2 + 5
      }
    );

    this.bottomLeftRect = createSvg(
      "rect",
      {
        class: "sw-resize",
        'data-id': "sw-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: 1,
        y: rect.height + 9
      }
    );

    this.bottomRect = createSvg(
      "rect",
      {
        class: "s-resize",
        'data-id': "s-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width / 2 + 5,
        y: rect.height + 9
      }
    );
    
    this.bottomRightRect = createSvg(
      "rect",
      {
        class: "se-resize",
        'data-id': "se-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width + 9,
        y: rect.height + 9
      }
    );

    this.rightRect = createSvg(
      "rect",
      {
        class: "e-resize",
        'data-id': "e-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width + 9,
        y: rect.height / 2 + 5
      }
    );

    this.topRightRect = createSvg(
      "rect",
      {
        class: "ne-resize",
        'data-id': "ne-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width + 9,
        y: 1
      }
    );

    this.topRect = createSvg(
      "rect",
      {
        class: "n-resize",
        'data-id': "n-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width / 2 + 5,
        y: 1
      }
    );

    this.outerLine.append(this.moveRect)
    this.outerLine.append(this.topLeftRect)
    this.outerLine.append(this.LeftRect)
    this.outerLine.append(this.bottomLeftRect)
    this.outerLine.append(this.bottomRect)
    this.outerLine.append(this.bottomRightRect)
    this.outerLine.append(this.rightRect)
    this.outerLine.append(this.topRightRect)
    this.outerLine.append(this.topRect)
    this.outerLineContainer.append(this.outerLine)
    this.outerLineContainer.append(this.deletetButton)
    this.initial = true

    if (this.layer.documentViewer.toolMode !== 'form') {
      if (this.annotation.isHidden == 1) {
        this.annotationContainer.style.display = 'none'
        this.annotationContainer.style.pointerEvents = 'none'
      } else if (this.annotation.isHidden == 0) {
        this.annotationContainer.style.display = 'block'
        this.annotationContainer.style.pointerEvents = 'auto'
      }
    }
    if (this.show) {
      !this.layer.addMode && this.selectAnnotation()
      this.show = false
    }
  }

  getActualRect (s) {
    const annotation = this.annotation
    const { left: x1, top: y1, right: x2, bottom: y2 } = annotation.rect
    
    const start = getActualPoint(
      {
        x: x1,
        y: y1
      },
      s
    )
    const end = getActualPoint(
      {
        x: x2,
        y: y2
      },
      s
    )
    return {
      start,
      end
    }
  }

  rectCalc ({ start, end }) {
    return {
      top: start.y < end.y ? start.y : end.y,
      left: start.x < end.x ? start.x : end.x,
      width: Math.abs(end.x - start.x),
      height: Math.abs(end.y - start.y),
    };
  };

  calculate (start, end) {
    const initRect = this.rectCalc({ start, end })
    this.initRect = initRect
    const actualbdwidth = (this.annotation.borderWidth || 2) * this.layer.viewport.scale

    return {
      top: initRect.top - actualbdwidth,
      left: initRect.left - actualbdwidth,
      width: initRect.width + actualbdwidth * 2,
      height: initRect.height + actualbdwidth * 2,
    }
  }

  getInitialPoint () {
    const s = this.layer.viewport.scale
    const startX = this.start.x
    const startY = this.start.y
    const endX = this.end.x
    const endY = this.end.y

    const { width, height, rotation  } = this.layer.viewport;
    let x1, y1, x2, y2;
    if (rotation === 0) {
      x1 = startX / s;
      y1 = startY / s; 
      x2 = endX / s;
      y2 = endY / s;
    } else if (rotation === 90) {
      x1 = startY / s;
      y1 = startX / s;
      x2 = endY / s;
      y2 = endX / s;
    } else if (rotation === 180) {
      x1 =( width - startX) / s;
      y1 = startY / s;
      x2 = (width - endX) / s;
      y2 = endY / s;
    } else {
      x1 = (height - startY) / s
      y1 = (width - startX) / s
      x2 = (height - endY) / s
      y2 = (width - endX)  / s
    }
    return {
      start: {
        x: x1,
        y: y1,
      },
      end: {
        x: x2,
        y: y2,
      },
    };
  }

  updateTool () {
    if (!this.initial) return
    if (this.hidden) {
      if (this.layer.annotationStore.selectedElementName === this.annotation.name) {
        this.layer.annotationStore.selectedElementName = null
      }
      this.outerLineContainer.remove()
    } else {
      if (this.layer.annotationStore.selectedElementName !== this.annotation.name) {
        const annotation = this.annotation
        this.layer.annotationStore.selectedElementName = annotation.name

        const annotationData = {
          ...annotation
        }
        delete annotationData.annotPtr
        delete annotationData.pagePtr
        delete annotationData.operate
        delete annotationData.targetPage

        this.layer.eventBus.dispatch('annotationSelected', annotationData)
      }
      this.layer.div.append(this.outerLineContainer)
      this.outerLine.addEventListener('mousedown', this.onMousedown)
      this.outerLine.addEventListener('touchstart', this.onMousedown)

      this.layer.eventBus.dispatch('propertyChange', this.annotation)

      const documentViewer = this.layer.documentViewer
      const scrollElement = documentViewer.getScrollViewElement()
      scrollElement.style.touchAction = 'none'
    }
  }

  handleClick (event) {
    if (!this.hidden || document.fullscreenElement || this.layer.annotationStore.creating || ['redaction', 'remove'].includes(this.layer.tool) || document.querySelector('.compare-document-container').contains(event?.target)) return
    if (this.layer.documentViewer.toolMode === 'form') {
      this.hidden = false
      this.updateTool()
      
      if (document.getElementsByClassName('right-panel')[0].className.indexOf('closed') !== -1) {
        document.getElementById('propertyPanelButton').click()
      }

      const ignoreList = [this.annotationContainer, this.outerLine, this.deletetButton, document.getElementById('propertyPanelButton'), document.getElementsByClassName('right-panel')[0]]

      this.layer.eventBus._on('pushButtonPropertyPanelChanged', this.handlePropertyPanel)
      onClickOutsideUp(ignoreList, this.handleOutside.bind(this))
    } else {
      if (this.annotation.actionType == 1 && this.annotation.destPage) {
        this.layer.eventBus.dispatch('pageNumberChanged', this.annotation.destPage)
      } else if (this.annotation.actionType == 6 && this.annotation.url) {
        this.buttonContainer.click()
      }
    }
  }

  selectAnnotation () {
    if (this.layer.selectedElementName === this.annotation.name) return
    this.handleClick()
  }

  handleOutside () {
    if (!this.hidden && this.layer.annotationStore.selectedElementName === this.annotation.name) {
      document.getElementById('propertyPanelButton').click()
      
      this.layer.eventBus.dispatch('propertyChange', null)
    }

    this.hidden = true

    if (this.layer.annotationStore.selectedElementName === this.annotation.name) {
      this.layer.annotationStore.selectedElementName = null
    }
    this.outerLine?.removeEventListener('mousedown', this.onMousedown)
    this.outerLine?.removeEventListener('touchstart', this.onMousedown)
    this.outerLineContainer?.remove()

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    scrollElement.style.touchAction = 'auto'

    this.layer.eventBus._off('pushButtonPropertyPanelChanged', this.handlePropertyPanel)
  }

  handleMouseDown (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    const operatorId = event.target.getAttribute('data-id')

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    this.scrollTop = scrollElement.scrollTop
    this.scrollLeft = scrollElement.scrollLeft

    const { pageX, pageY } = getClickPoint(event)
    this.startState = {
      operator: operatorId,
      clickX: pageX,
      clickY: pageY,
    }

    document.addEventListener('mousemove', this.onMousemove)
    document.addEventListener('mouseup', this.onMouseup)
    document.addEventListener('touchmove', this.onMousemove)
    document.addEventListener('touchend', this.onMouseup)

    this.handleClick()
  };

  handleMouseMove (event) {
    if (event.type === 'touchmove') {
      document.querySelector('.document-container').addEventListener('scroll', event => event.preventDefault())
    }
    this.moving = true
    let { pageX, pageY } = getClickPoint(event)

    const documentViewer = this.layer.documentViewer
    const scrollElement = documentViewer.getScrollViewElement()
    const scrollTop = scrollElement.scrollTop
    const scrollLeft = scrollElement.scrollLeft
    pageX = pageX + scrollLeft - this.scrollLeft
    pageY = pageY + scrollTop - this.scrollTop

    const start = this.start
    const end = this.end
    const startState = this.startState

    const { width, height } = this.layer.viewport
    if (startState.operator === 'nw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(end.y - 5, y1)
      y1 = Math.max(0, y1)

      this.update({
        start: { x: x1, y: y1 },
        end,
      });
    } else if (startState.operator === 'w-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)

      this.update({
        start: { x: x1, y: start.y },
        end,
      });
    } else if (startState.operator === 'sw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.end.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(height, y1)
      y1 = Math.max(start.y + 5, y1)

      this.update({
        start: { x: x1, y: start.y },
        end: { x: end.x, y: y1 },
      });
    } else if (startState.operator === 's-resize') {
      let y1 = pageY - (startState.clickY - this.end.y);
      y1 = Math.min(height, y1)
      y1 = Math.max(start.y + 5, y1)
      this.update({
        start,
        end: { x: end.x, y: y1 },
      });
    } else if (startState.operator === 'se-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.end.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(height, y)
      y = Math.max(start.y + 5, y)
      this.update({
        start,
        end: { x, y },
      });
    } else if (startState.operator === 'e-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      this.update({
        start,
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'ne-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.start.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(end.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start: { x: start.x, y },
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'n-resize') {
      let y = pageY - (startState.clickY - this.start.y);
      y = Math.min(end.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start: { x: start.x, y },
        end,
      });
    } else if (startState.operator === 'move') {
      let x1, y1, x2, y2
      const documentViewer = this.layer.documentViewer

      let windowCoordinates = this.getMouseLocation(event);
      const page = documentViewer.getSelectedPage(windowCoordinates, windowCoordinates)
      if (page.first && page.first !== (this.layer.page + 1)) {
        const pagePoint = documentViewer.windowToPage(windowCoordinates, page.first, false)

        const annotationWidth = Math.abs(start.x - end.x)
        const annotationHeight = Math.abs(start.y - end.y)
        if (documentViewer.pdfViewer.spreadMode === 0 && documentViewer.pdfViewer.scrollMode === 0) {
          x1 = pageX - (startState.clickX - this.start.x)
          y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
          x2 = pageX - (startState.clickX - this.end.x)
          y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
        } else if (documentViewer.pdfViewer.spreadMode === 0 && documentViewer.pdfViewer.scrollMode === 1) {
          x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
          y1 = pageY - (startState.clickY - this.start.y)
          x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
          y2 = pageY - (startState.clickY - this.end.y)
        } else if (documentViewer.pdfViewer.spreadMode === 1 && documentViewer.pdfViewer.scrollMode === 0) {
          const minPage = Math.min(page.first, this.layer.page + 1)
          const maxPage = Math.max(page.first, this.layer.page + 1)
          if ((maxPage - minPage) > 2 || ((minPage + 1) === maxPage && minPage % 2 === 0)) {
            return
          }
          if ((minPage + 1) === maxPage) {
            x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
            y1 = pageY - (startState.clickY - this.start.y)
            x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
            y2 = pageY - (startState.clickY - this.end.y)
          } else if ((minPage + 2) === maxPage) {
            x1 = pageX - (startState.clickX - this.start.x)
            y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
            x2 = pageX - (startState.clickX - this.end.x)
            y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
          }
        } else if (documentViewer.pdfViewer.spreadMode === 2 && documentViewer.pdfViewer.scrollMode === 0) {
          const minPage = Math.min(page.first, this.layer.page + 1)
          const maxPage = Math.max(page.first, this.layer.page + 1)
          if ((maxPage - minPage) > 2 || ((minPage + 1) === maxPage && !(minPage === 1 && maxPage === 2) && maxPage % 2 === 0 || (minPage === 1 && maxPage === 3))) {
            return
          }
          if ((minPage + 2) === maxPage || (minPage === 1 && maxPage === 2)) {
            x1 = pageX - (startState.clickX - this.start.x)
            y1 = page.first > (this.layer.page + 1) ? pagePoint.y : pagePoint.y - annotationHeight
            x2 = pageX - (startState.clickX - this.end.x)
            y2 = page.first > (this.layer.page + 1) ? pagePoint.y + annotationHeight : pagePoint.y
          } else if ((minPage + 1) === maxPage) {
            x1 = page.first > (this.layer.page + 1) ? pagePoint.x : pagePoint.x - annotationWidth
            y1 = pageY - (startState.clickY - this.start.y)
            x2 = page.first > (this.layer.page + 1) ? pagePoint.x + annotationWidth : pagePoint.x
            y2 = pageY - (startState.clickY - this.end.y)
          }
        }
        this.layer = documentViewer.pdfViewer._pages[page.first - 1].compdfAnnotationLayer

        this.annotationContainer.remove()
        this.outerLineContainer.remove()
        const container = this.layer.div
        container.append(this.annotationContainer)
        container.append(this.outerLineContainer)
      } else {
        x1 = pageX - (startState.clickX - this.start.x);
        y1 = pageY - (startState.clickY - this.start.y);
        x2 = pageX - (startState.clickX - this.end.x);
        y2 = pageY - (startState.clickY - this.end.y);

        const windowPoint1 = documentViewer.pageToWindow({
          x: x1,
          y: y1
        }, this.layer.page + 1, false)
        const windowPoint2 = documentViewer.pageToWindow({
          x: x2,
          y: y2
        }, this.layer.page + 1, false)
        let x = 0, y = 0
        if (pageY > startState.clickY) {
          y = scrollElement.offsetHeight - (windowPoint2.y - scrollElement.scrollTop - scrollElement.offsetTop)
          y = y < 100 ? 5 : 0
        } else if (pageY < startState.clickY) {
          y = windowPoint1.y - scrollElement.scrollTop - scrollElement.offsetTop
          y = y < 100 ? -5 : 0
        }
        if (pageX > startState.clickX) {
          x = scrollElement.offsetWidth - (windowPoint2.x - scrollElement.scrollLeft - scrollElement.offsetLeft)
          x = x < 100 ? 5 : 0
        } else if (pageX < startState.clickX) {
          x = (windowPoint1.x - scrollElement.scrollLeft - scrollElement.offsetLeft)
          x = x < 100 ? -5 : 0
        }

        if (x || y) {
          scrollElement.scrollTop += y
          scrollElement.scrollLeft += x
        }

        const rect = {
          width: Math.abs(start.x -end.x),
          height: Math.abs(start.y -end.y)
        }
        if (x1 < x2) {
          x1 = Math.max(0, x1)
          x1 = Math.min(width - rect.width, x1)
          x2 = Math.max(rect.width, x2)
          x2 = Math.min(width, x2)
        } else {
          x2 = Math.max(0, x2)
          x2 = Math.min(width - rect.width, x2)
          x1 = Math.max(rect.width, x1)
          x1 = Math.min(width, x1)
        }

        if (y1 < y2) {
          y1 = Math.max(0, y1)
          y1 = Math.min(height - rect.height, y1)
          y2 = Math.max(rect.height, y2)
          y2 = Math.min(height, y2)
        } else {
          y2 = Math.max(0, y2)
          y2 = Math.min(height - rect.height, y2)
          y1 = Math.max(rect.height, y1)
          y1 = Math.min(height, y1)
        }
      }
      this.update({
        start: { x: x1, y: y1 },
        end: { x: x2, y: y2 },
      });
    }

    startState.clickX = pageX
    startState.clickY = pageY
  }

  handleDelete (event) {
    if (this.layer.tool && event) {
      event.stopPropagation()
    }

    this.delete()
    const annotationData = {
      type: 'delete',
      annotation: {
        operate: "del-annot",
        name: this.annotation.name,
        pageIndex: this.layer.page
      }
    }
    this.annotation.annotPtr && (annotationData.annotation.annotPtr = this.annotation.annotPtr)
    this.layer.eventBus.dispatch('annotationChanged', annotationData)
  }

  delete () {
    this.handleOutside()
    this.annotationContainer?.remove()
    this.annotation.isDelete = true
    this.layer.deleteAnnotationInLayer({
      name: this.annotation.name
    })
  }

  handleMouseUp (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    if (event.type === 'touchend') {
      document.querySelector('.document-container').style.overflow = 'auto'
    }
    this.moving = false
    document.removeEventListener('mousemove', this.onMousemove)
    document.removeEventListener('mouseup', this.onMouseup)
    document.removeEventListener('touchmove', this.onMousemove)
    document.removeEventListener('touchend', this.onMouseup)
    this.scrollLeft = 0
    this.scrollTop = 0

    const annotation = this.annotation
    const { start, end } = this.getInitialPoint()

    const rect = {
      left: start.x,
      top: start.y,
      right: end.x,
      bottom: end.y
    }
    if (rect === annotation.rect && annotation.pageIndex === this.layer.page) return

    annotation.rect = rect
    if (annotation.pageIndex !== this.layer.page) {
      const annotationData = {
        type: 'delete',
        annotation: {
          operate: "del-annot",
          name: annotation.name,
          pageIndex: annotation.pageIndex,
          annotPtr: annotation.annotPtr
        }
      }
      this.layer.eventBus.dispatch('annotationChanged', annotationData)

      const newAnnotationData = Object.assign({}, annotation)
      delete newAnnotationData.annotPtr
      delete newAnnotationData.pagePtr
      delete newAnnotationData.name
      newAnnotationData.operate = 'add-form'
      newAnnotationData.pageIndex = this.layer.page
      this.layer.eventBus.dispatch('annotationChanged', {
        type: 'add',
        show: true,
        annotation: newAnnotationData
      })

      this.annotationContainer.remove()
      this.outerLineContainer.remove()
    } else {
      this.layer.eventBus.dispatch('annotationChanged', {
        type: 'modify',
        annotation: {
          operate: "mod-form",
          name: annotation.name,
          pageIndex: this.layer.page,
          pagePtr: annotation.pagePtr,
          annotPtr: annotation.annotPtr,
          rect
        }
      })
      this.getImage(true)
    }
  }

  update ({ start, end }) {
    const rect = this.calculate(start, end)

    this.start = start
    this.end = end

    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'
    
    this.annotationContainer.style.width = `${rect.width}px`;
    this.annotationContainer.style.height = `${rect.height}px`;

    this.outerLine.style.left = `${rect.left - 8}px`
    this.outerLine.style.top = `${rect.top - 8}px`
    this.outerLine.style.width = `${rect.width + 8 * 2}px`
    this.outerLine.style.height = `${rect.height + 8 * 2}px`

    const left = (rect.left + rect.width - 30) + 'px'
    const top = (rect.top + rect.height + 8) + 'px'
    this.deletetButton.style.left = left
    this.deletetButton.style.top = top
    
    this.moveRect.setAttribute('width',  rect.width + 9)
    this.moveRect.setAttribute('height',  rect.height + 9)
    
    this.LeftRect.setAttribute("y", rect.height / 2 + 5)

    this.bottomLeftRect.setAttribute("y", rect.height + 9)

    this.bottomRect.setAttribute("x", rect.width / 2 + 5)
    this.bottomRect.setAttribute("y", rect.height + 9)


    this.bottomRightRect.setAttribute("x", rect.width + 9)
    this.bottomRightRect.setAttribute("y", rect.height + 9)


    this.rightRect.setAttribute("x", rect.width + 9)
    this.rightRect.setAttribute("y", rect.height / 2 + 5)

    
    this.topRightRect.setAttribute("x", rect.width + 9)

    this.topRect.setAttribute("x", rect.width / 2 + 5)
  }

  handlePropertyPanel (props) {
    if (this.layer.annotationStore.selectedElementName !== this.annotation.name) return
    const annotation = this.annotation

    let updatePropsNum = 0
    for (const key in props) {
      for (const item in this.annotation) {
        if (item === key && this.annotation[item] !== props[key]) {
          this.annotation[item] = props[key]
          updatePropsNum ++

          if (key === 'backgroundColor') {
            this.buttonContainer.style[key] = props[key]
            if (props[key] === '') {
              props.clearBackgroundColor = true
            }
          }
          if (key === 'fontSize') {
            this.buttonContainer.style.fontSize = props[key] + 'px'
          }
          if (key === 'color') {
            this.buttonContainer.style.color = props[key]
          }
          if (key === 'fontName') {
            this.buttonContainer.style.fontFamily = props[key]
          }
          if (key === 'title') {
            this.buttonContainer.innerHTML = props[key]
          }
          if (key === 'url') {
            this.buttonContainer.setAttribute('href', this.annotation.url)
            this.buttonContainer.setAttribute('target', '_blank')
          }
          if (key === 'fontStyle') {
            const fontStyle = props[key]
            if (fontStyle === 'boldItalic') {
              this.buttonContainer.style.fontStyle = 'italic'
              this.buttonContainer.style.fontWeight = 'bold'
              annotation.isBold = true
              annotation.isItalic = true
            } else if (fontStyle === 'bold') {
              this.buttonContainer.style.fontStyle = 'normal'
              this.buttonContainer.style.fontWeight = 'bold'
              annotation.isBold = true
              annotation.isItalic = false
            } else if (fontStyle === 'italic') {
              this.buttonContainer.style.fontStyle = 'italic'
              this.buttonContainer.style.fontWeight = 'normal'
              annotation.isBold = false
              annotation.isItalic = true
            } else {
              annotation.isItalic = false
              annotation.isBold = false
              this.buttonContainer.style.fontStyle = 'normal'
              this.buttonContainer.style.fontWeight = 'normal'
            }
          }
        }
      }
    }

    if (updatePropsNum > 0) {
      const annotationData = {
        operate: "mod-form",
        name: this.annotation.name,
        pageIndex: this.layer.page,
        pagePtr: this.annotation.pagePtr,
        annotPtr: this.annotation.annotPtr,
        pageHeight: this.layer.viewport.rawDims.pageHeight
      }
      if ((props.destPage || props.url) && this.annotation.actionType !== '0') {
        annotationData.actionType = this.annotation.actionType
      }

      const isFont = ['fontSize', 'fontName', 'color', 'fontStyle'].some(item => props[item])
      if (isFont) {
        annotationData.fontSize = annotation.fontSize
        annotationData.color = annotation.color
        const fontArr = {
          Courier: {
            normal: 'Courier',
            bold: 'Courier-Bold',
            italic: 'Courier-Oblique',
            boldItalic: 'Courier-BoldOblique'
          },
          Helvetica: {
            normal: 'Helvetica',
            bold: 'Helvetica-Bold',
            italic: 'Helvetica-Oblique',
            boldItalic: 'Helvetica-BoldOblique'
          },
          Times: {
            normal: 'Times',
            bold: 'Times-Bold',
            italic: 'Times-Italic',
            boldItalic: 'Times-BoldItalic'
          }
        }

        const fontName = annotation.fontName
        if (annotation.isBold && annotation.isItalic) {
          annotationData.fontName = fontArr[fontName].boldItalic
        } else if (annotation.isBold) {
          annotationData.fontName = fontArr[fontName].bold
        } else if (annotation.isItalic) {
          annotationData.fontName = fontArr[fontName].italic
        } else {
          annotationData.fontName = fontArr[fontName] ? fontArr[fontName].normal : fontName
        }
        delete props.fontName
      }

      const updateAnnot = Object.assign({}, annotationData, props)
      this.layer.eventBus.dispatch('annotationChanged', {
        type: 'modify',
        annotation: updateAnnot
      })
      this.getImage()
    }
  }

  setProperty (props) {
    if (props.name !== this.annotation.name) return
    const annotation = this.annotation

    let updatePropsNum = 0
    for (const key in props) {
      for (const item in this.annotation) {
        if (item === key && this.annotation[item] !== props[key]) {
          this.annotation[item] = props[key]
          updatePropsNum ++

          if (key === 'backgroundColor') {
            this.buttonContainer.style[key] = props[key]
            if (props[key] === '') {
              props.clearBackgroundColor = true
            }
          }
          if (key === 'fontSize') {
            this.buttonContainer.style.fontSize = props[key] + 'px'
          }
          if (key === 'color') {
            this.buttonContainer.style.color = props[key]
          }
          if (key === 'fontName') {
            this.buttonContainer.style.fontFamily = props[key]
          }
          if (key === 'title') {
            this.buttonContainer.innerHTML = props[key]
          }
          if (key === 'url') {
            this.buttonContainer.setAttribute('href', this.annotation.url)
            this.buttonContainer.setAttribute('target', '_blank')
          }
          if (key === 'fontStyle') {
            const fontStyle = props[key]
            if (fontStyle === 'boldItalic') {
              this.buttonContainer.style.fontStyle = 'italic'
              this.buttonContainer.style.fontWeight = 'bold'
              annotation.isBold = true
              annotation.isItalic = true
            } else if (fontStyle === 'bold') {
              this.buttonContainer.style.fontStyle = 'normal'
              this.buttonContainer.style.fontWeight = 'bold'
              annotation.isBold = true
              annotation.isItalic = false
            } else if (fontStyle === 'italic') {
              this.buttonContainer.style.fontStyle = 'italic'
              this.buttonContainer.style.fontWeight = 'normal'
              annotation.isBold = false
              annotation.isItalic = true
            } else {
              annotation.isItalic = false
              annotation.isBold = false
              this.buttonContainer.style.fontStyle = 'normal'
              this.buttonContainer.style.fontWeight = 'normal'
            }
          }
        }
      }
    }

    if (updatePropsNum > 0) {
      const annotationData = {
        operate: "mod-form",
        name: this.annotation.name,
        pageIndex: this.layer.page,
        pagePtr: this.annotation.pagePtr,
        annotPtr: this.annotation.annotPtr,
        pageHeight: this.layer.viewport.rawDims.pageHeight
      }
      if ((props.destPage || props.url) && this.annotation.actionType !== '0') {
        annotationData.actionType = this.annotation.actionType
      }

      const isFont = ['fontSize', 'fontName', 'color', 'fontStyle'].some(item => props[item])
      if (isFont) {
        annotationData.fontSize = annotation.fontSize
        annotationData.color = annotation.color
        const fontArr = {
          Courier: {
            normal: 'Courier',
            bold: 'Courier-Bold',
            italic: 'Courier-Oblique',
            boldItalic: 'Courier-BoldOblique'
          },
          Helvetica: {
            normal: 'Helvetica',
            bold: 'Helvetica-Bold',
            italic: 'Helvetica-Oblique',
            boldItalic: 'Helvetica-BoldOblique'
          },
          Times: {
            normal: 'Times',
            bold: 'Times-Bold',
            italic: 'Times-Italic',
            boldItalic: 'Times-BoldItalic'
          }
        }

        const fontName = annotation.fontName
        if (annotation.isBold && annotation.isItalic) {
          annotationData.fontName = fontArr[fontName].boldItalic
        } else if (annotation.isBold) {
          annotationData.fontName = fontArr[fontName].bold
        } else if (annotation.isItalic) {
          annotationData.fontName = fontArr[fontName].italic
        } else {
          annotationData.fontName = fontArr[fontName] ? fontArr[fontName].normal : fontName
        }
        delete props.fontName
      }

      const updateAnnot = Object.assign({}, annotationData, props)
      this.layer.eventBus.dispatch('annotationChanged', {
        type: 'modify',
        annotation: updateAnnot
      })
      this.getImage()
    }
  }

  setHighlight (bool) {
    this.highlight = bool
    this.annotationContainer.style.backgroundColor = this.highlight ? '#4982E54D' : ''
  }

  async getImage(update) {
    const rect = this.initRect
    const imgRect = {
      width: parseInt(rect.width * this.layer.ratio + 1),
      height: parseInt(rect.height * this.layer.ratio + 1)
    }

    const imageArray = await this.layer.messageHandler.sendWithPromise('GetRenderAnnot', {
      annotPtr: this.annotation.annotPtr,
      width: imgRect.width,
      height: imgRect.height
    })

    const canvas = document.createElement('canvas')
    canvas.style.width = imgRect.width + 'px'
    canvas.style.height = imgRect.height + 'px'
    canvas.width = imgRect.width
    canvas.height = imgRect.height
    let drawContext = canvas.getContext("2d")
    drawContext.clearRect(0, 0, canvas.width, canvas.height)
    let imageData = drawContext.createImageData(imgRect.width, imgRect.height)
    imageData.data.set(imageArray)
    drawContext.putImageData(imageData, 0, 0)
    const imgSrc = canvas.toDataURL("image/png", 1)

    if (update) {
      const rect = this.calculate(this.start, this.end)
      this.shapeElement.style.width = `${rect.width}px`
      this.shapeElement.style.height = `${rect.height}px`
    }

    if (this.imgEle) {
      this.imgEle.src = imgSrc
    } else {
      let imgEle = document.createElement('img')
      imgEle.style.width = '100%'
      imgEle.style.height = '100%'
      imgEle.style.pointerEvents = 'none'
      imgEle.style.display = 'block'

      imgEle.src = imgSrc
      this.imgEle = imgEle

      this.shapeElement.append(this.imgEle)
    }
  }
}
