import MakupAnnotation from './markup_annotation'
import { onClickOutside, convertColorToRGB } from '../ui_utils'
import { createSvg, createElement } from '../annotation/utils'
import { getActualPoint } from '../annotation/utils'

class TextHighlightAnnotation extends MakupAnnotation {
  constructor ({
    annotation,
    layer,
    enableReply = true,
  }) {
    super({
      annotation,
      layer,
    })
    this.type = 'highlight'
    this.enableReply = enableReply
    this.hidden = true
    this.markupContainer = null
    this.outerLineContainer = null
    this.ratio = window.devicePixelRatio || 1
    this.layer.pageViewer.canvas && (this.ctx = this.layer.pageViewer.canvas.getContext('2d', { willReadFrequently: true }))
    this.lineWidth = 2 * this.layer.viewport.scale * this.ratio

    this.onHandleClick = this.handleClick.bind(this)
    this.onDelete = this.handleDelete.bind(this)
    this.onOpenReply = this.openReply.bind(this)

    this.render()
  }

  drawCanvas(positionArray) {
    const annotation = this.annotation
    const { R, G, B } = convertColorToRGB(annotation.color)
    this.ctx.fillStyle = `rgba(${R}, ${G}, ${B}, ${annotation.opacity || 1})`
    for (let i = 0; i < positionArray.length; i++) {
      const position = positionArray[i]
      this.ctx.fillRect(position.left * this.ratio, position.top * this.ratio, position.width * this.ratio, position.height * this.ratio)
    }
  }
}

export default TextHighlightAnnotation
