import MakupAnnotation from './markup_annotation'
import { convertColorToRGB } from '../ui_utils'

class TextSquigglyAnnotation extends MakupAnnotation {
  constructor ({
    annotation,
    layer,
    enableReply = true,
  }) {
    super({
      annotation,
      layer,
    })
    this.type = 'squiggly'
    this.enableReply = enableReply
    this.hidden = true
    this.markupContainer = null
    this.outerLineContainer = null
    this.ratio = window.devicePixelRatio || 1
    this.layer.pageViewer.canvas && (this.ctx = this.layer.pageViewer.canvas.getContext('2d', { willReadFrequently: true }))
    this.lineWidth = 2 * this.layer.viewport.scale * this.ratio

    this.onHandleClick = this.handleClick.bind(this)
    this.onDelete = this.handleDelete.bind(this)
    this.onOpenReply = this.openReply.bind(this)

    this.render()
  }

  drawCanvas(positionArray) {
    const annotation = this.annotation
    const amplitude = 3 * this.layer.viewport.scale  // 波动幅度
    const wavelength = 20 * this.layer.viewport.scale // 波长
    const { R, G, B } = convertColorToRGB(annotation.color)
    this.ctx.strokeStyle = `rgba(${R}, ${G}, ${B}, ${annotation.opacity || 1})`
    this.ctx.lineWidth = this.lineWidth

    for (let i = 0; i < positionArray.length; i++) {
      const position = positionArray[i]
      this.ctx.beginPath()
      for (let x = position.left * this.ratio; x <= (position.left + position.width) * this.ratio; x++) {
        const y = (position.top + position.height) * this.ratio + amplitude * Math.sin((x / wavelength) * 2 * Math.PI)
        this.ctx.lineTo(x, y)
      }
      this.ctx.stroke()
    }
  }
}

export default TextSquigglyAnnotation
